@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: if-mib-proc.h,v 1.2 2002/02/19 07:45:09 schoenw Exp $
+ * @(#) $Id: if-mib-proc.h,v 1.5 2002/07/19 09:11:03 schoenw Exp $
  */
 
 #ifndef _IF_MIB_PROC_H_
@@ -35,24 +35,12 @@
  */
 
 extern void
-if_mib_proc_set_interface_status(GSnmpSession *s,
-				 gint32 ifIndex,
-				 gint32 ifAdminStatus);
+if_mib_proc_get_ifTable(GSnmpSession *s,
+			if_mib_ifEntry_t ***ifEntry,
+			gint mask,
+			time_t epoch);
 
 extern void
-if_mib_proc_set_interface_alias(GSnmpSession *s,
-				gint32 ifIndex,
-				guchar *alias,
-				gsize alias_len);
+if_mib_proc_free_ifTable(if_mib_ifEntry_t **ifEntry);
 
-extern void
-if_mib_proc_set_interface_promiscuous(GSnmpSession *s,
-				      gint32 ifIndex,
-				      gint32 ifPromiscuousMode);
-
-extern void
-if_mib_proc_set_notifications(GSnmpSession *s,
-			      gint32 ifIndex,
-			      gint32 ifLinkUpDownTrapEnable);
-    
 #endif /* _IF_MIB_PROC_H_ */
