@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: rapid-city-proc.h,v 1.1 2001/09/30 20:33:47 schoenw Exp $
+ * @(#) $Id: rapid-city-proc.h,v 1.2 2002/06/25 14:31:51 schoenw Exp $
  */
 
 #ifndef _RAPID_CITY_PROC_H_
@@ -30,7 +30,7 @@
 
 extern void
 rapid_city_proc_create_vlan(GSnmpSession *s, gint32 vlanid,
-			    guchar *name, guint32 type);
+			    guchar *name, gsize name_len, guint32 type);
 extern void
 rapid_city_proc_delete_vlan(GSnmpSession *s, gint32 vlanid);
 
