@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: basic.c,v 1.61 2002/03/22 16:13:50 schoenw Exp $
+ * @(#) $Id: basic.c,v 1.73 2002/07/19 12:02:41 schoenw Exp $
  */
 
 #include "scli.h"
@@ -46,8 +46,8 @@ struct error_info {
 
 static const struct error_info error_infos[] = {
     { SCLI_MSG,			"%s" },
-    { SCLI_OK,			"ok" },
-    { SCLI_EXIT,		"ok" },
+    { SCLI_OK,			"ok; scli " VERSION " ready" },
+    { SCLI_EXIT,		"ok; scli " VERSION " exiting" },
     { SCLI_ERROR,		"%s" },
     { SCLI_ERROR_NOPEER,	"no association to a remote SNMP agent" },
     { SCLI_ERROR_NOXML,		"command `%s' does not support xml" },
@@ -58,7 +58,7 @@ static const struct error_info error_infos[] = {
     { SCLI_SYNTAX_VALUE,	"invalid value" },
     { SCLI_SYNTAX_TOKENIZER,	NULL },
     { SCLI_SYNTAX_COMMAND,	NULL },
-    { SCLI_SNMP,		NULL },
+    { SCLI_SNMP,		"SNMP communication error (timeout)" },
     { SCLI_SNMP_NAME,		"name lookup failed" },
     { 0, NULL }
 };
@@ -222,11 +222,12 @@ page(scli_interp_t *interp, GString *s)
 
 
 scli_interp_t*
-scli_interp_create()
+scli_interp_create(char *name)
 {
     scli_interp_t *interp;
 
-    interp = g_malloc0(sizeof(scli_interp_t));
+    interp = g_new0(scli_interp_t, 1);
+    interp->name = name ? g_strdup(name) : NULL;
     interp->cmd_root = g_node_new(NULL);
     interp->result = g_string_new(NULL);
     interp->header = g_string_new(NULL);
@@ -243,6 +244,9 @@ void
 scli_interp_delete(scli_interp_t *interp)
 {
     if (interp) {
+	if (interp->name) {
+	    g_free(interp->name);
+	}
 	if (interp->cmd_root) {
 	    g_node_destroy(interp->cmd_root);
 	}
@@ -264,6 +268,33 @@ scli_interp_delete(scli_interp_t *interp)
 
 
 
+void scli_interp_init(scli_interp_t *interp)
+{
+    scli_init_scli_mode(interp);
+    scli_init_system_mode(interp);
+    scli_init_entity_mode(interp);
+    scli_init_disman_mode(interp);
+    scli_init_snmp_mode(interp);
+    scli_init_ip_mode(interp);
+    scli_init_udp_mode(interp);
+    scli_init_tcp_mode(interp);
+    scli_init_interface_mode(interp);
+    scli_init_bridge_mode(interp);
+    scli_init_atm_mode(interp);
+    scli_init_ether_mode(interp);
+    scli_init_ospf_mode(interp);
+    scli_init_printer_mode(interp);
+    scli_init_isdn_mode(interp);
+    scli_init_rs232_mode(interp);
+    scli_init_sonet_mode(interp);
+    scli_init_cisco_mode(interp);
+    scli_init_nortel_mode(interp);
+    scli_init_netsnmp_mode(interp);
+    scli_init_3com_mode(interp);
+}
+
+
+
 void
 scli_interp_reset(scli_interp_t *interp)
 {
@@ -302,7 +333,7 @@ scli_alarm_create(scli_interp_t *interp, char *desc)
 {
     scli_alarm_t *alarm;
 
-    alarm = g_malloc0(sizeof(scli_alarm_t));
+    alarm = g_new0(scli_alarm_t, 1);
     alarm->detected = time(NULL);
     alarm->verified = alarm->detected;
     if (desc) alarm->desc = g_strdup(desc);
@@ -493,7 +524,7 @@ expand_alias(scli_interp_t *interp, char *cmd)
     if (token) {
 	for (elem = interp->alias_list; elem; elem = g_slist_next(elem)) {
 	    scli_alias_t *alias = (scli_alias_t *) elem->data;
-	    if (strcmp(alias->name, token) == 0) {
+	    if (strcasecmp(alias->name, token) == 0) {
 		new_cmd = g_strdup_printf("%s %s",
 					  alias->value, rest ? rest : "");
 		return new_cmd;
@@ -535,14 +566,18 @@ scli_create_command(scli_interp_t *interp, scli_cmd_t *cmd)
 	     child;
 	     child = g_node_next_sibling(child)) {
 	    scli_cmd_t *this_cmd = (scli_cmd_t *) child->data;
-	    if (strcmp(this_cmd->name, argv[i]) == 0) {
+	    if (strcasecmp(this_cmd->name, argv[i]) == 0) {
 		break;
 	    }
 	}
 	if (! child) {
 	    scli_cmd_t *newcmd;
-	    newcmd = (scli_cmd_t *) g_malloc0(sizeof(scli_cmd_t));
+	    int j;
+	    newcmd = g_new0(scli_cmd_t, 1);
 	    newcmd->name = g_strdup(argv[i]);
+	    for (j = 0; newcmd->name[j]; j++) {
+		newcmd->name[j] = tolower(newcmd->name[j]);
+	    }
 	    child = g_node_append(node, g_node_new(newcmd));
 	}
 	node = child;
@@ -648,16 +683,22 @@ show_xxx(scli_interp_t *interp, scli_cmd_t *cmd, int code)
     case SCLI_SYNTAX_VALUE:
     case SCLI_SYNTAX_NUMBER:
     case SCLI_SYNTAX_REGEXP:
-	reason = g_strdup_printf(error_infos[i].fmt);
+	if (interp->result->len) {
+	    reason = g_strdup_printf("%s \"%s\"", error_infos[i].fmt,
+				     interp->result->str);
+	} else {
+	    reason = g_strdup(error_infos[i].fmt);
+	}
 	break;
     case SCLI_SYNTAX_NUMARGS:
-	reason = g_strdup_printf("wrong number of arguments: should be `%s%s%s'",
-		cmd->path,
-		cmd->options ? " " : "",
-		cmd->options ? cmd->options : "");
+	reason = g_strdup_printf("wrong number of arguments: "
+				 "should be `%s%s%s'",
+				 cmd->path,
+				 cmd->options ? " " : "",
+				 cmd->options ? cmd->options : "");
 	break;
     case SCLI_SYNTAX:
-	reason = g_strdup_printf("%3d usage: %s %s", code, cmd->path,
+	reason = g_strdup_printf("usage: %s %s", cmd->path,
 				 cmd->options ? cmd->options : "");
 	break;
     case SCLI_SNMP:
@@ -665,33 +706,33 @@ show_xxx(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 	    const char *error;
 	    error = gsnmp_enum_get_label(gsnmp_enum_error_table,
 					 interp->peer->error_status);
-	    reason = g_strdup_printf("%s", error ? error : "internalError");
-#if 0
 	    if ((int) (interp->peer->error_status) > 0) {
-		g_print("@%d", interp->peer->error_index);
+		reason = g_strdup_printf("%s @ varbind %d",
+					 error ? error : "internalError",
+					 interp->peer->error_index);
+	    } else {
+		reason = g_strdup(error ? error : "internalError");
 	    }
-#endif
 	} else {
-	    reason = g_strdup("SNMP communication error (timeout)");
+	    reason = g_strdup(error_infos[i].fmt);
 	}
 	break;
-
     case SCLI_SNMP_NAME:
-	reason = g_strdup_printf("%s", error_infos[i].fmt);
+	reason = g_strdup(error_infos[i].fmt);
 	break;
     case SCLI_ERROR:
 	reason = g_strdup_printf(error_infos[i].fmt,
 			 interp->result->str ? interp->result->str : "");
 	break;
-    case SCLI_ERROR_NOPEER:
-	reason = g_strdup_printf(error_infos[i].fmt);
+    case SCLI_ERROR_NOPEER: 
+	reason = g_strdup(error_infos[i].fmt);
 	break;
     case SCLI_ERROR_NOXML:
 	reason = g_strdup_printf(error_infos[i].fmt, cmd->path);
 	break;
     case SCLI_OK:
     case SCLI_EXIT:
-	reason = g_strdup("ok");
+	reason = g_strdup(error_infos[i].fmt);
 	break;
     }
 
@@ -701,26 +742,37 @@ show_xxx(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 	}
 	if (interp->xml_doc && interp->xml_doc->children) {
 	    xmlNodePtr top = interp->xml_doc->children;
+#if 0
 	    xml_set_prop(top, "code", "%3d", code);
 	    if (reason) {
 		xml_set_prop(top, "reason", "%s", reason);
 	    }
+#endif
 	    if (interp->peer) {
 		xml_set_prop(top, "peer", interp->peer->taddress);
 	    }
 	    xml_set_prop(top, "date", "%s", fmt_timeticks(0));
 	}
 	if (! (scli_interp_recursive(interp))) {
-	    show_result(interp, code);
+	    if (interp->xml_doc->children
+		&& interp->xml_doc->children->children) {
+		show_result(interp, code);
+	    }
+	    if (scli_interp_proto(interp)) {
+		g_printerr("%3d %s\n", code, reason);
+	    }
 	}
     } else {
 	if (code == SCLI_OK || code == SCLI_EXIT) {
 	    if (! (scli_interp_recursive(interp))
 		&& ! (cmd->flags & SCLI_CMD_FLAG_MONITOR)) {
 		show_result(interp, code);
+		if (scli_interp_proto(interp)) {
+		    g_printerr("%3d %s\n", code, reason);
+		}
 	    }
 	} else {
-	    g_printerr("%s\n", reason);
+	    g_printerr("%3d %s\n", code, reason);
 	}
     }
 
@@ -748,6 +800,7 @@ eval_all_cmd_node(scli_interp_t *interp, GNode *node, GString *s)
 	if (G_NODE_IS_LEAF(node)) {
 	    scli_cmd_t *cmd = node->data;
 	    if (cmd
+		&& !(cmd->flags & SCLI_CMD_FLAG_NORECURSE)
 		&& !(cmd->flags & SCLI_CMD_FLAG_NEED_PEER && !interp->peer)) {
 		code = scli_eval_cmd(interp, cmd, 0, NULL);
 		if (code == SCLI_OK && interp->result->len) {
@@ -847,12 +900,13 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
     int i, code = SCLI_OK;
 
     scli_interp_reset(interp);
+    interp->xid++;
 
     node = g_node_first_child(interp->cmd_root);
     for (i = 0; i < argc && ! done; i++) {
 	while (node) {
 	    scli_cmd_t *cmd = (scli_cmd_t *) node->data;
-	    if (cmd && strcmp(cmd->name, argv[i]) == 0) {
+	    if (cmd && strcasecmp(cmd->name, argv[i]) == 0) {
 		break;
 	    }
 	    node = g_node_next_sibling(node);
@@ -886,6 +940,9 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
 	    g_string_assign(interp->result, s->str);
 	    g_string_free(s, 1);
 	    show_result(interp, code);
+	    if (scli_interp_proto(interp)) {
+		g_printerr("%3d xxx\n", code);
+	    }
 	    scli_interp_reset(interp);
 	    interp->flags &= ~SCLI_INTERP_FLAG_RECURSIVE;
 	}
@@ -993,11 +1050,15 @@ scli_eval_file(scli_interp_t *interp, char *path)
     g_return_val_if_fail(interp, SCLI_ERROR);
     g_return_val_if_fail(path, SCLI_ERROR);
 
-    stream = fopen(path, "r");
-    if (! stream) {
-	g_print("%3d failed to open file \"%s\": %s\n", SCLI_ERROR,
-		path, g_strerror(errno));
-	return SCLI_ERROR;
+    if (strcmp(path, "-") == 0) {
+	stream = stdin;
+    } else {
+	stream = fopen(path, "r");
+	if (! stream) {
+	    g_print("%3d failed to open file \"%s\": %s\n", SCLI_ERROR,
+		    path, g_strerror(errno));
+	    return SCLI_ERROR;
+	}
     }
     code = scli_eval_file_stream(interp, stream);
     (void) fclose(stream);
@@ -1036,6 +1097,7 @@ scli_open_community(scli_interp_t *interp, char *host, int port,
 		    char *community)
 {
     snmpv2_mib_system_t *system = NULL;
+    int verbose;
 
     if (interp->peer) {
 	scli_close(interp);
@@ -1055,34 +1117,45 @@ scli_open_community(scli_interp_t *interp, char *host, int port,
     /*
      * Lets see how we can talk to this guy. We first try to speek
      * SNMPv2c (since this protocol does much better error handling)
-     * and we fall back to SNMPv1 only if this is necessary.
+     * and we fall back to SNMPv1 only if this is necessary. However,
+     * if the user specified a specific version, we trust him and
+     * do no probe at all.
      */
 
     g_snmp_list_decode_hook = NULL;
 
-    if (scli_interp_interactive(interp)) {
-	g_print("%3d Trying SNMPv2c ... ", SCLI_MSG);
-    }
-    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
-    if (interp->peer->error_status == 0) {
-	if (scli_interp_interactive(interp)) {
-	    g_print("good!\n");
-	}
+    verbose = scli_interp_interactive(interp) && !scli_interp_quiet(interp);
+
+    if (interp->snmp == G_SNMP_V1
+	|| interp->snmp == G_SNMP_V2C
+	|| interp->snmp == G_SNMP_V3) {
+	interp->peer->version = interp->snmp;
     } else {
-	if (scli_interp_interactive(interp)) {
-	    g_print("timeout.\n%d Trying SNMPv1  ... ", SCLI_MSG);
+	if (verbose) {
+	    g_print("%3d-scli trying SNMPv2c ... ", SCLI_MSG);
 	}
-	interp->peer->version = G_SNMP_V1;
+	interp->peer->version = G_SNMP_V2C;
 	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
 	if (interp->peer->error_status == 0) {
-	    if (scli_interp_interactive(interp)) {
-		g_print("ok.\n");
+	    if (verbose) {
+		g_print("good\n");
 	    }
 	} else {
-	    if (scli_interp_interactive(interp)) {
-		g_print("timeout.\n");
+	    if (verbose) {
+		g_print("timeout\n%3d-scli trying SNMPv1  ... ", SCLI_MSG);
+	    }
+	    interp->peer->version = G_SNMP_V1;
+	    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
+	    if (interp->peer->error_status == 0) {
+		if (verbose) {
+		    g_print("ok.\n");
+		}
+	    } else {
+		if (verbose) {
+		    g_print("timeout.\n");
+		}
+		scli_close(interp);
 	    }
-	    scli_close(interp);
 	}
     }
     if (system) {
@@ -1117,6 +1190,10 @@ char*
 scli_prompt(scli_interp_t *interp)
 {
     char *prompt;
+
+    if (scli_interp_proto(interp)) {
+	return NULL;
+    }
     
     if (interp->peer) {
 	prompt = g_strdup_printf("(%s) scli > ", interp->peer->taddress);
