@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: bridge.c,v 1.35 2002/01/08 19:02:47 schoenw Exp $
+ * @(#) $Id: bridge.c,v 1.41 2002/07/03 18:41:16 schoenw Exp $
  */
 
 #include "scli.h"
@@ -87,7 +87,9 @@ fmt_stpStatus(GString *s, gint32 *enable, gint32 *state)
 
 
 static void
-fmt_bridge_stp_info(GString *s, bridge_mib_dot1dStp_t *dot1dStp)
+fmt_bridge_stp_info(GString *s,
+		    bridge_mib_dot1dBase_t *dot1dBase,
+		    bridge_mib_dot1dStp_t *dot1dStp)
 {
     int const indent = 18;
     const char *e;
@@ -101,15 +103,28 @@ fmt_bridge_stp_info(GString *s, bridge_mib_dot1dStp_t *dot1dStp)
     }
     
     if (dot1dStp->dot1dStpPriority) {
-	g_string_sprintfa(s, "%-*s%d (%02x:%02x)\n", indent,
+	g_string_sprintfa(s, "%-*s%d (0x%04x)\n", indent,
 			      "Stp Priority:",
 			  *dot1dStp->dot1dStpPriority,
+			  *dot1dStp->dot1dStpPriority);
+    }
+
+    if (dot1dStp->dot1dStpPriority && dot1dBase->dot1dBaseBridgeAddress) {
+	g_string_sprintfa(s, "%-*s%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n", indent,
+			  "Stp Bridge ID:",
 			  (*dot1dStp->dot1dStpPriority >> 8) & 0xff,
-			  (*dot1dStp->dot1dStpPriority) & 0xff);
+			  (*dot1dStp->dot1dStpPriority) & 0xff,
+			  dot1dBase->dot1dBaseBridgeAddress[0],
+			  dot1dBase->dot1dBaseBridgeAddress[1],
+			  dot1dBase->dot1dBaseBridgeAddress[2],
+			  dot1dBase->dot1dBaseBridgeAddress[3],
+			  dot1dBase->dot1dBaseBridgeAddress[4],
+			  dot1dBase->dot1dBaseBridgeAddress[5]);
+	
     }
     
     if (dot1dStp->dot1dStpDesignatedRoot) {
-	g_string_sprintfa(s, "%-*s%s\n", indent, "Stp Root:",
+	g_string_sprintfa(s, "%-*s%s\n", indent, "Stp Root ID:",
 			  fmt_bridgeid(dot1dStp->dot1dStpDesignatedRoot));
     }
 
@@ -124,8 +139,34 @@ fmt_bridge_stp_info(GString *s, bridge_mib_dot1dStp_t *dot1dStp)
     }
 
     if (dot1dStp->dot1dStpMaxAge) {
-	g_string_sprintfa(s, "%-*s%d seconds\n", indent, "Stp Aging Time:",
-			  *dot1dStp->dot1dStpMaxAge);
+	g_string_sprintfa(s, "%-*s%d milliseconds\n", indent,
+			  "Stp Aging Time:", *dot1dStp->dot1dStpMaxAge * 10);
+    }
+
+    if (dot1dStp->dot1dStpHelloTime) {
+	g_string_sprintfa(s, "%-*s%d milliseconds\n", indent,
+			  "Stp Hello Time:", *dot1dStp->dot1dStpHelloTime * 10);
+    }
+
+    if (dot1dStp->dot1dStpHoldTime) {
+	g_string_sprintfa(s, "%-*s%d milliseconds\n", indent,
+			  "Stp Hold  Time:", *dot1dStp->dot1dStpHoldTime * 10);
+    }
+
+    if (dot1dStp->dot1dStpForwardDelay) {
+	g_string_sprintfa(s, "%-*s%d milliseconds\n", indent,
+			  "Stp Fwd Delay:", *dot1dStp->dot1dStpForwardDelay * 10);
+    }
+
+    if (dot1dStp->dot1dStpTimeSinceTopologyChange) {
+	g_string_sprintfa(s, "%-*s", indent, "Stp Topo Change:");
+	g_string_append(s, fmt_timeticks(*(dot1dStp->dot1dStpTimeSinceTopologyChange)));
+	g_string_append_c(s, '\n');
+    }
+    
+    if (dot1dStp->dot1dStpTopChanges) {
+	g_string_sprintfa(s, "%-*s%u (estimated)\n", indent,
+			  "Stp Topo Changes:", *dot1dStp->dot1dStpTopChanges);
     }
     
 }
@@ -173,7 +214,7 @@ fmt_bridge_info(GString *s,
     }
 
     if (dot1dStp) {
-	fmt_bridge_stp_info(s, dot1dStp);
+	fmt_bridge_stp_info(s, dot1dBase, dot1dStp);
     }
 }
 
@@ -360,7 +401,9 @@ fmt_bridge_stp_port(GString *s,
 		  dot1dStpPortEntry->dot1dStpPortState);
 
     if (dot1dStpPortEntry->dot1dStpPortPathCost) {
-	g_string_sprintfa(s, "%5u ", *dot1dStpPortEntry->dot1dStpPortPathCost);
+	g_string_sprintfa(s, "%6u  ", *dot1dStpPortEntry->dot1dStpPortPathCost);
+    } else {
+	g_string_sprintfa(s, "%6s  ", "");
     }
 
     if (dot1dStpPortEntry->dot1dStpPortDesignatedRoot) {
@@ -529,27 +572,28 @@ fmt_bridge_filter(GString *s,
 	g_string_sprintfa(s, "%5d ",
 			  dot1dStaticEntry->dot1dStaticReceivePort);
     } else {
-	g_string_sprintfa(s, "%5s ", "");
-    }
-
-    if (dot1dStaticEntry->dot1dStaticAddress) {
-	char *addr = fmt_ether_address(dot1dStaticEntry->dot1dStaticAddress,
-				       SCLI_FMT_ADDR);
-	g_string_append(s, addr);
-    } else {
-	g_string_sprintfa(s, "--:--:--:--:--:--");
-    }
-
-    if (dot1dStaticEntry->dot1dStaticAllowedToGoTo) {
-	/* xxx */
+	g_string_sprintfa(s, "%5s ", "*");
     }
 
     status = fmt_enum(bridge_mib_enums_dot1dStaticStatus,
 		      dot1dStaticEntry->dot1dStaticStatus);
     if (status) {
-	g_string_sprintfa(s, " %s", status);
+	g_string_sprintfa(s, "%-15s ", status);
     }
     
+    if (dot1dStaticEntry->dot1dStaticAddress) {
+	char *addr = fmt_ether_address(dot1dStaticEntry->dot1dStaticAddress,
+				       SCLI_FMT_ADDR);
+	g_string_sprintfa(s, "%s ", addr);
+    } else {
+	g_string_sprintfa(s, "--:--:--:--:--:-- ");
+    }
+
+    if (dot1dStaticEntry->dot1dStaticAllowedToGoTo) {
+	fmt_port_set(s, dot1dStaticEntry->dot1dStaticAllowedToGoTo,
+		     dot1dStaticEntry->_dot1dStaticAllowedToGoToLength);
+    }
+
     g_string_append(s, "\n");
 }
 
@@ -577,6 +621,8 @@ show_bridge_filter(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (dot1dStaticTable) {
+	g_string_sprintfa(interp->header,
+			  " PORT STATUS          ADDRESS           ALLOWED-PORTS");
 	for (i = 0; dot1dStaticTable[i]; i++) {
 	    fmt_bridge_filter(interp->result, dot1dStaticTable[i]);
 	}
@@ -669,7 +715,7 @@ show_bridge_stats(scli_interp_t *interp, int argc, char **argv)
     if (! stats && portTable) {
 	for (i = 0; portTable[i]; i++) ;
 	stats_size = i;
-	stats = (bridge_stats_t *) g_malloc0(i * sizeof(bridge_stats_t));
+	stats = g_new0(bridge_stats_t, i);
 	for (i = 0; portTable[i]; i++) {
 	    stats[i].descr = get_port_description(interp,
 						  portTable[i]->dot1dTpPort);
