@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: disman.c,v 1.38 2002/01/18 09:45:44 schoenw Exp $
+ * @(#) $Id: disman.c,v 1.41 2002/06/26 09:20:29 schoenw Exp $
  */
 
 #include "scli.h"
@@ -160,62 +160,6 @@ fmt_last_change(GString *s, guchar *date, gsize len)
 
 
 
-static void
-fmt_storage_type(GString *s, gint32 *storage)
-{
-    static GSnmpEnum const storage_types[] = {
-	{ 1, "o" },	/* other */
-	{ 2, "v" },	/* volatile */
-	{ 3, "n" },	/* nonVolatile */
-	{ 4, "p" },	/* permanent */
-	{ 5, "r" },	/* readOnly */
-	{ 0, NULL }
-    };
-
-    const char *label;
-
-    if (! storage) {
-	g_string_append(s, " ");
-	return;
-    }
-
-    label = gsnmp_enum_get_label(storage_types, *storage);
-    if (label) {
-	g_string_append(s, label);
-    } else {
-	g_string_append(s, "-");
-    }
-}
-
-
-
-static void
-fmt_row_status(GString *s, gint32 *status)
-{
-    static GSnmpEnum const row_states[] = {
-	{ 1, "A" },	/* active */
-	{ 2, "S" },	/* notInService */
-	{ 3, "R" },	/* notReady */
-	{ 0, NULL }
-    };
-
-    const char *label;
-
-    if (! status) {
-	g_string_append(s, " ");
-	return;
-    }
-
-    label = gsnmp_enum_get_label(row_states, *status);
-    if (label) {
-	g_string_append(s, label);
-    } else {
-	g_string_append(s, "-");
-    }
-}
-
-
-
 static void
 fmt_script_admin_status(GString *s, gint32 *status)
 {
@@ -1612,9 +1556,14 @@ create_disman_script(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (strlen(argv[1]) > 32
-	|| strlen(argv[2]) < 1
-	|| strlen(argv[2]) > 32) {
+    if (strlen(argv[1]) < DISMAN_SCRIPT_MIB_SMSCRIPTOWNERMINLENGTH
+	|| strlen(argv[1]) > DISMAN_SCRIPT_MIB_SMSCRIPTOWNERMAXLENGTH) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_VALUE;
+    }
+    if (strlen(argv[2]) < DISMAN_SCRIPT_MIB_SMSCRIPTNAMEMINLENGTH
+	|| strlen(argv[2]) > DISMAN_SCRIPT_MIB_SMSCRIPTNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1638,9 +1587,15 @@ create_disman_run(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (strlen(argv[1]) > 32
-	|| strlen(argv[2]) < 1
-	|| strlen(argv[2]) > 32) {
+    if (strlen(argv[1]) < DISMAN_SCRIPT_MIB_SMLAUNCHOWNERMINLENGTH
+	|| strlen(argv[1]) > DISMAN_SCRIPT_MIB_SMLAUNCHOWNERMAXLENGTH) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_VALUE;
+    }
+
+    if (strlen(argv[2]) < DISMAN_SCRIPT_MIB_SMLAUNCHNAMEMINLENGTH
+	|| strlen(argv[2]) > DISMAN_SCRIPT_MIB_SMLAUNCHNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1662,13 +1617,13 @@ scli_init_disman_mode(scli_interp_t *interp)
 
 	{ "create disman script", "<owner> <name> <description>",
 	  "...",
-	  SCLI_CMD_FLAG_NEED_PEER,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  create_disman_script },	  
 
 	{ "create disman run", "<owner> <name> <args>",
 	  "...",
-	  SCLI_CMD_FLAG_NEED_PEER,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  create_disman_run },	  
 
