@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: interface.c,v 1.66 2002/02/19 07:37:09 schoenw Exp $
+ * @(#) $Id: interface.c,v 1.78 2002/08/14 08:28:12 schoenw Exp $
  */
 
 #include "scli.h"
@@ -141,8 +141,20 @@ fmt_ifStatus(GString *s, gint32 *admin, gint32 *oper,
 
 
 
-static int
-match_interface(regex_t *regex_iface,
+static void
+fmt_phys_address(GString *s, guchar *address, guint16 address_len)
+{
+     int i;
+
+     for (i = 0; i < address_len; i++) {
+	  g_string_sprintfa(s, "%s%02X", (i == 0) ? "" : ":", address[i]);
+     }
+}
+
+
+
+int
+interface_match(regex_t *regex_iface,
 		if_mib_ifEntry_t *ifEntry)
 {
     int status;
@@ -289,12 +301,14 @@ fmt_interface_details(GString *s,
 		      if_mib_ifEntry_t *ifEntry,
 		      if_mib_ifXEntry_t *ifXEntry,
 		      snmpv2_mib_system_t *system,
-		      ip_mib_ipAddrEntry_t **ipAddrTable)
+		      ip_mib_ipAddrEntry_t **ipAddrTable,
+		      if_mib_ifRcvAddressEntry_t **ifRcvAddrTable)
 {
     int j;
     char *name;
     int const width = 20;
     const char *e;
+    const char *f;
 
     g_string_sprintfa(s, "Interface:   %-*d", width,
 		      ifEntry->ifIndex);
@@ -310,10 +324,8 @@ fmt_interface_details(GString *s,
     g_string_sprintfa(s, "OperStatus:  %-*s", width, e ? e : "");
     if (ifEntry->ifPhysAddress && ifEntry->_ifPhysAddressLength) {
 	g_string_append(s, " Address: ");
-	for (j = 0; j < ifEntry->_ifPhysAddressLength; j++) {
-	    g_string_sprintfa(s, "%s%02X", (j == 0) ? "" : ":",
-			      ifEntry->ifPhysAddress[j]);
-	}
+	fmt_phys_address(s, ifEntry->ifPhysAddress,
+			 ifEntry->_ifPhysAddressLength);
 	/* See RFC 2665 section 3.2.6. why the test below is so ugly... */
 	if (ifEntry->ifType && ifEntry->_ifPhysAddressLength == 6
 	    && (*ifEntry->ifType == IF_MIB_IFTYPE_ETHERNETCSMACD
@@ -372,6 +384,28 @@ fmt_interface_details(GString *s,
 	g_string_append(s, " Change:\n");
     }
 
+    if (ifRcvAddrTable) {
+	for (j = 0; ifRcvAddrTable[j]; j++) {
+	    if (ifRcvAddrTable[j]->ifIndex == ifEntry->ifIndex
+		&& ifRcvAddrTable[j]->ifRcvAddressAddress) {
+		e = fmt_enum(if_mib_enums_ifRcvAddressStatus,
+			     ifRcvAddrTable[j]->ifRcvAddressStatus);
+		f = fmt_enum(if_mib_enums_ifRcvAddressType,
+			     ifRcvAddrTable[j]->ifRcvAddressType);
+		g_string_sprintfa(s, "RecvStatus:  %s/%s",
+				  e ? e : "", f ? f : "");
+		g_string_sprintfa(s, "%*s", width - 1 -
+				  (e ? strlen(e) : 0) - (f ? strlen(f) : 0),
+				  "");
+		g_string_sprintfa(s, " Address: ");
+		fmt_phys_address(s,
+				 ifRcvAddrTable[j]->ifRcvAddressAddress,
+				 ifRcvAddrTable[j]->_ifRcvAddressAddressLength);
+		g_string_sprintfa(s, "\n");
+	    }
+	}
+    }
+
     if (ipAddrTable) {
 	for (j = 0; ipAddrTable[j]; j++) {
 	    if (ipAddrTable[j]->ipAdEntIfIndex
@@ -408,7 +442,8 @@ xml_interface_details(xmlNodePtr root,
 		      if_mib_ifEntry_t *ifEntry,
 		      if_mib_ifXEntry_t *ifXEntry,
 		      snmpv2_mib_system_t *system,
-		      ip_mib_ipAddrEntry_t **ipAddrTable)
+		      ip_mib_ipAddrEntry_t **ipAddrTable,
+		      if_mib_ifRcvAddressEntry_t **ifRcvAddrTable)
 {
     int j;
     xmlNodePtr tree, node;
@@ -576,6 +611,7 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 {
     if_mib_ifEntry_t **ifTable = NULL;
     if_mib_ifXEntry_t **ifXTable = NULL;
+    if_mib_ifRcvAddressEntry_t **ifRcvAddrTable = NULL;
     snmpv2_mib_system_t *system = NULL;
     ip_mib_ipAddrEntry_t **ipAddrTable = NULL;
     entity_mib_entAliasMappingEntry_t **entAliasMappingTable = NULL;
@@ -593,6 +629,7 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_iface = &_regex_iface;
 	if (regcomp(regex_iface, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -602,7 +639,8 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_OK;
     }
 
-    if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFENTRY_PARAMS);
+    if_mib_proc_get_ifTable(interp->peer, &ifTable,
+			    IF_MIB_IFENTRY_PARAMS, interp->epoch);
     if (interp->peer->error_status) {
 	if (regex_iface) regfree(regex_iface);
 	return SCLI_SNMP;
@@ -610,7 +648,7 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 
     if (ifTable) {
 	if_mib_get_ifXTable(interp->peer, &ifXTable, IF_MIB_IFXENTRY_PARAMS);
-	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
+	if_mib_get_ifRcvAddressTable(interp->peer, &ifRcvAddrTable, 0);
 	ip_mib_get_ipAddrTable(interp->peer, &ipAddrTable, 0);
 	
 	entity_mib_get_entAliasMappingTable(interp->peer,
@@ -619,13 +657,15 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 	    entity_mib_get_entPhysicalTable(interp->peer, &entPhysicalTable,
 					    ENTITY_MIB_ENTPHYSICAL_MASK);
 	}
+ 	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
 	for (i = 0, c = 0; ifTable[i]; i++) {
-	    if (match_interface(regex_iface, ifTable[i])) {
+	    if (interface_match(regex_iface, ifTable[i])) {
 		if_mib_ifXEntry_t *ifXEntry;
 		ifXEntry = get_ifXEntry(ifXTable, ifTable[i]->ifIndex);
 		if (scli_interp_xml(interp)) {
 		    xml_interface_details(interp->xml_node, ifTable[i],
-					  ifXEntry, system, ipAddrTable);
+					  ifXEntry, system, ipAddrTable,
+					  ifRcvAddrTable);
 		} else {
 		    if (c) {
 			g_string_append(interp->result, "\n");
@@ -633,7 +673,8 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 		    entPhysicalEntry = get_entity(ifTable[i]->ifIndex,
 				  entAliasMappingTable, entPhysicalTable);
 		    fmt_interface_details(interp->result, ifTable[i],
-					  ifXEntry, system, ipAddrTable);
+					  ifXEntry, system, ipAddrTable,
+					  ifRcvAddrTable);
 		    if (entPhysicalEntry) {
 			fmt_entity_root(interp, entPhysicalTable,
 					entPhysicalEntry);
@@ -644,8 +685,9 @@ show_interface_details(scli_interp_t *interp, int argc, char **argv)
 	}
     }
 
-    if (ifTable) if_mib_free_ifTable(ifTable);
+    if (ifTable) if_mib_proc_free_ifTable(ifTable);
     if (ifXTable) if_mib_free_ifXTable(ifXTable);
+    if (ifRcvAddrTable) if_mib_free_ifRcvAddressTable(ifRcvAddrTable);
     if (system) snmpv2_mib_free_system(system);
     if (ipAddrTable) ip_mib_free_ipAddrTable(ipAddrTable);
     if (entAliasMappingTable)
@@ -734,6 +776,7 @@ show_interface_info(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_iface = &_regex_iface;
 	if (regcomp(regex_iface, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -743,7 +786,8 @@ show_interface_info(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_OK;
     }
 
-    if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFENTRY_PARAMS);
+    if_mib_proc_get_ifTable(interp->peer, &ifTable,
+			    IF_MIB_IFENTRY_PARAMS, interp->epoch);
     if (interp->peer->error_status) {
 	if (regex_iface) regfree(regex_iface);
 	return SCLI_SNMP;
@@ -758,7 +802,7 @@ show_interface_info(scli_interp_t *interp, int argc, char **argv)
 			  type_width, "TYPE",
 			  name_width, "NAME");
 	for (i = 0; ifTable[i]; i++) {
-	    if (match_interface(regex_iface, ifTable[i])) {
+	    if (interface_match(regex_iface, ifTable[i])) {
 		if_mib_ifXEntry_t *ifXEntry;
 		ifXEntry = get_ifXEntry(ifXTable, ifTable[i]->ifIndex);
 		fmt_interface_info(interp->result, ifTable[i],
@@ -774,7 +818,7 @@ show_interface_info(scli_interp_t *interp, int argc, char **argv)
 	}
     }
 
-    if (ifTable) if_mib_free_ifTable(ifTable);
+    if (ifTable) if_mib_proc_free_ifTable(ifTable);
     if (ifXTable) if_mib_free_ifXTable(ifXTable);
 
     if (regex_iface) regfree(regex_iface);
@@ -785,55 +829,147 @@ show_interface_info(scli_interp_t *interp, int argc, char **argv)
 
 
 static void
-fmt_interface_stack(GString *s,
-		    if_mib_ifStackEntry_t *ifStackEntry,
-		    if_mib_ifStackEntry_t **ifStackTable,
-		    if_mib_ifEntry_t **ifTable,
-		    int level, int type_width)
+fmt_interface_stack_line(GString *s, if_mib_ifEntry_t *ifEntry,
+			 int type_width, const char *o)
 {
-    /*
-     *      ,-- if2 
-     * if1 -+-- if3
-     *      `-- if4
-     *
-     * if5 -,
-     * if6 -+-- if8
-     * if7 -'
-     */
-
-    int i = 0;
     const char *e;
-    char *ifDescr = NULL;
-    int ifDescrLength = 0;
-    if_mib_ifEntry_t *ifEntry = NULL;
 
-    if (! ifStackEntry->ifStackStatus
-	|| *ifStackEntry->ifStackStatus != IF_MIB_IFSTACKSTATUS_ACTIVE) {
-	return;
+    g_string_sprintfa(s, "%9u %s ", ifEntry->ifIndex, o);
+    e = fmt_enum(if_mib_enums_ifType, ifEntry->ifType);
+    g_string_sprintfa(s, "%-*s ", type_width, e ? e : "");
+    if (ifEntry->ifDescr) {
+	g_string_sprintfa(s, "%.*s",
+			  (int) ifEntry->_ifDescrLength,
+			  ifEntry->ifDescr);
+    }
+    g_string_append(s, "\n");
+}
+
+
+
+static void
+fmt_interface_stack(scli_interp_t *interp,
+		    if_mib_ifEntry_t **ifTable,
+		    if_mib_ifStackEntry_t **ifStackTable,
+		    regex_t *regex_iface, int type_width)
+{
+    int i, j;
+    
+    /*
+     * First check for all 1:1 stacks...
+     */
+    
+    for (i = 0; ifTable[i]; i++) {
+	if_mib_ifEntry_t *ifEntry = NULL;
+	int l = 0, h = 0;
+	
+	for (j = 0; ifStackTable[j]; j++) {
+	    if (ifStackTable[j]->ifStackLowerLayer == ifTable[i]->ifIndex
+		&& ifStackTable[j]->ifStackHigherLayer) {
+		if (! ifEntry) {
+		    ifEntry = get_ifEntry(ifTable,
+					  ifStackTable[j]->ifStackHigherLayer);
+		    if (! ifEntry) {
+			g_warning("unknown stacked interface #%d",
+				  ifStackTable[j]->ifStackHigherLayer);
+			continue;
+			}
+		}
+		h++;
+	    }
+	}
+	if (h == 1 && ifEntry) {
+	    for (j = 0; ifStackTable[j]; j++) {
+		if (ifStackTable[j]->ifStackHigherLayer == ifEntry->ifIndex
+		    && ifStackTable[j]->ifStackLowerLayer) {
+		    l++;
+		}
+	    }
+	}
+	if (h == 1 && l == 1 && ifTable[i] && ifEntry) {
+	    if (interface_match(regex_iface, ifTable[i])
+		|| interface_match(regex_iface, ifEntry)) {
+		fmt_interface_stack_line(interp->result, ifTable[i],
+					 type_width, "-.   ");
+		fmt_interface_stack_line(interp->result, ifEntry,
+					 type_width, "  `->");
+	    }
+	}
     }
     
-    ifEntry = get_ifEntry(ifTable, ifStackEntry->ifStackHigherLayer);
-
-    if (ifEntry->ifDescr) {
-	ifDescr = ifEntry->ifDescr;
-	ifDescrLength = ifEntry->_ifDescrLength;
-    }
-
-    g_string_sprintfa(s, "%9d ", ifStackEntry->ifStackHigherLayer);
-    e = fmt_enum(if_mib_enums_ifType, ifEntry->ifType);
-    g_string_sprintfa(s, "%-*s", type_width, e ? e : "");
-    g_string_sprintfa(s, " %*s%s %.*s\n",
-		      level*4, "", level ? "`-" : " -",
-		      ifDescrLength, ifDescr ? ifDescr : "");
-
-    for (i = 0; ifStackTable[i]; i++) {
-	if ((ifStackTable[i]->ifStackLowerLayer
-	     == ifStackEntry->ifStackHigherLayer)
-	    && ifStackTable[i]->ifStackHigherLayer) {
-	    fmt_interface_stack(s, ifStackTable[i], ifStackTable, ifTable,
-				level+1, type_width);
-	    if (ifStackTable[i]->ifStackStatus) {
-		*ifStackTable[i]->ifStackStatus = 0;
+    /*
+     * Check for all 1:n and n:1 stacks with n > 1.
+     */
+    
+    for (i = 0; ifTable[i]; i++) {
+	int l = 0, h = 0;
+	if (! interface_match(regex_iface, ifTable[i])) {
+	    continue;
+	}
+	for (j = 0; ifStackTable[j]; j++) {
+	    if (ifStackTable[j]->ifStackLowerLayer == ifTable[i]->ifIndex
+		&& ifStackTable[j]->ifStackHigherLayer) {
+		h++;
+	    }
+	    if (ifStackTable[j]->ifStackHigherLayer == ifTable[i]->ifIndex
+		&& ifStackTable[j]->ifStackLowerLayer) {
+		l++;
+	    }
+	}
+	if (h > 1) {
+	    int cnt = 0;
+	    for (j = 0; ifStackTable[j]; j++) {
+		if (ifStackTable[j]->ifStackLowerLayer == ifTable[i]->ifIndex
+		    && ifStackTable[j]->ifStackHigherLayer) {
+		    if_mib_ifEntry_t *ifEntry;
+		    ifEntry = get_ifEntry(ifTable,
+					  ifStackTable[j]->ifStackHigherLayer);
+		    if (! ifEntry) {
+			g_warning("unknown stacked interface #%d",
+				  ifStackTable[j]->ifStackHigherLayer);
+			continue;
+		    }
+		    if (cnt == 0) {
+			fmt_interface_stack_line(interp->result, ifTable[i],
+						 type_width, "--.  ");
+		    }
+		    if (cnt+1 == h) {
+			fmt_interface_stack_line(interp->result, ifEntry,
+						 type_width, "  `->");
+		    } else {
+			fmt_interface_stack_line(interp->result, ifEntry,
+						 type_width, "  |->");
+		    }
+		    cnt++;
+		}
+	    }
+	}
+	if (l > 1) {
+	    int cnt = 0;
+	    for (j = 0; ifStackTable[j]; j++) {
+		if (ifStackTable[j]->ifStackHigherLayer == ifTable[i]->ifIndex
+		    && ifStackTable[j]->ifStackLowerLayer) {
+		    if_mib_ifEntry_t *ifEntry;
+		    ifEntry = get_ifEntry(ifTable,
+					  ifStackTable[j]->ifStackLowerLayer);
+		    if (! ifEntry) {
+			g_warning("unknown stacked interface #%d",
+				  ifStackTable[j]->ifStackLowerLayer);
+			continue;
+		    }
+		    if (cnt == 0) {
+			fmt_interface_stack_line(interp->result, ifEntry,
+						 type_width, "--.  ");
+		    } else {
+			fmt_interface_stack_line(interp->result, ifEntry,
+						 type_width, "--|  ");
+		    }
+		    if (cnt == l-1) {
+			fmt_interface_stack_line(interp->result, ifTable[i],
+						 type_width, "  `->");
+		    }
+		    cnt++;
+		}
 	    }
 	}
     }
@@ -846,7 +982,6 @@ show_interface_stack(scli_interp_t *interp, int argc, char **argv)
 {
     if_mib_ifStackEntry_t **ifStackTable = NULL;
     if_mib_ifEntry_t **ifTable = NULL;
-    if_mib_ifEntry_t *ifEntry = NULL;
     regex_t _regex_iface, *regex_iface = NULL;
     int i, type_width;
 
@@ -857,6 +992,7 @@ show_interface_stack(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_iface = &_regex_iface;
 	if (regcomp(regex_iface, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -873,31 +1009,31 @@ show_interface_stack(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (ifStackTable) {
-	if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFENTRY_PARAMS);
-	type_width = get_if_type_width(ifTable);
 
-	g_string_sprintfa(interp->header, "INTERFACE %-*s    STACKING ORDER",
-			  type_width, "TYPE");
+	/*
+	 * Check whether there is anything interesting to report -
+	 * otherwise we quit early saving some resources.
+	 */
+	
 	for (i = 0; ifStackTable[i]; i++) {
-	    if (ifStackTable[i]->ifStackLowerLayer == 0) {
-		ifEntry = get_ifEntry(ifTable,
-				      ifStackTable[i]->ifStackHigherLayer);
-		if (! ifEntry) {
-		    g_warning("stacked interface %d does not exist "
-			      "in the ifTable",
-			      ifStackTable[i]->ifStackHigherLayer);
-		    continue;
-		}
-		if (match_interface(regex_iface, ifEntry)) {
-		    fmt_interface_stack(interp->result, ifStackTable[i],
-					ifStackTable, ifTable, 0, type_width);
-		}
-	    }
+	    if (ifStackTable[i]->ifStackLowerLayer
+		&& ifStackTable[i]->ifStackHigherLayer) break;
+	}
+
+	if (ifStackTable[i]) {
+	    if_mib_proc_get_ifTable(interp->peer, &ifTable,
+				    IF_MIB_IFENTRY_PARAMS, interp->epoch);
+	    type_width = get_if_type_width(ifTable);
+	    g_string_sprintfa(interp->header,
+			      "INTERFACE STACK %-*s DESCRIPTION",
+			      type_width, "TYPE");
+	    fmt_interface_stack(interp, ifTable, ifStackTable,
+				regex_iface, type_width);
 	}
     }
 
     if (ifStackTable) if_mib_free_ifStackTable(ifStackTable);
-    if (ifTable) if_mib_free_ifTable(ifTable);
+    if (ifTable) if_mib_proc_free_ifTable(ifTable);
     
     if (regex_iface) regfree(regex_iface);
 
@@ -936,6 +1072,7 @@ show_interface_stats(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_iface = &_regex_iface;
 	if (regcomp(regex_iface, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -960,7 +1097,7 @@ show_interface_stats(scli_interp_t *interp, int argc, char **argv)
 
     if (! stats && ifTable) {
 	for (i = 0; ifTable[i]; i++) ;
-	stats = (if_stats_t *) g_malloc0(i * sizeof(if_stats_t));
+	stats = g_new0(if_stats_t, i);
     }
 
     epoch = time(NULL);
@@ -973,7 +1110,7 @@ show_interface_stats(scli_interp_t *interp, int argc, char **argv)
 	for (i = 0; ifTable[i]; i++) {
 	    GString *s;
 	    if_mib_ifXEntry_t *ifXEntry;
-	    if (! match_interface(regex_iface, ifTable[i])) {
+	    if (! interface_match(regex_iface, ifTable[i])) {
 		continue;
 	    }
 	    s = interp->result;
@@ -1078,6 +1215,7 @@ foreach_interface(scli_interp_t *interp, char *regex,
     int i;
 
     if (regcomp(regex_iface, regex, interp->regex_flags) != 0) {
+	g_string_assign(interp->result, regex);
 	return SCLI_SYNTAX_REGEXP;
     }
 
@@ -1086,7 +1224,8 @@ foreach_interface(scli_interp_t *interp, char *regex,
 	return SCLI_OK;
     }
     
-    if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFDESCR);
+    if_mib_proc_get_ifTable(interp->peer, &ifTable,
+			    IF_MIB_IFDESCR, interp->epoch);
     if (interp->peer->error_status) {
 	regfree(regex_iface);
 	return SCLI_SNMP;
@@ -1094,13 +1233,13 @@ foreach_interface(scli_interp_t *interp, char *regex,
 
     if (ifTable) {
 	for (i = 0; ifTable[i]; i++) {
-	    if (match_interface(regex_iface, ifTable[i])) {
+	    if (interface_match(regex_iface, ifTable[i])) {
 		(func) (interp, ifTable[i], user_data);
 	    }
 	}
     }
     
-    if (ifTable) if_mib_free_ifTable(ifTable);
+    if (ifTable) if_mib_proc_free_ifTable(ifTable);
     regfree(regex_iface);
 
     return SCLI_OK;
@@ -1114,7 +1253,7 @@ set_status(scli_interp_t *interp, if_mib_ifEntry_t *ifEntry,
 {
     gint32 *value = (gint32 *) user_data;
 
-    if_mib_proc_set_interface_status(interp->peer, ifEntry->ifIndex, *value);
+    if_mib_set_ifAdminStatus(interp->peer, ifEntry->ifIndex, *value);
     if (interp->peer->error_status) {
 	interface_snmp_error(interp, ifEntry);
     }
@@ -1135,6 +1274,7 @@ set_interface_status(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (! gsnmp_enum_get_number(if_mib_enums_ifAdminStatus, argv[2], &value)) {
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1148,8 +1288,8 @@ static void
 set_alias(scli_interp_t *interp, if_mib_ifEntry_t *ifEntry,
 	  gpointer user_data)
 {
-    if_mib_proc_set_interface_alias(interp->peer, ifEntry->ifIndex,
-				    (char *) user_data, strlen((char *) user_data));
+    if_mib_set_ifAlias(interp->peer, ifEntry->ifIndex,
+		       (char *) user_data, strlen((char *) user_data));
     if (interp->peer->error_status) {
 	interface_snmp_error(interp, ifEntry);
     }
@@ -1180,7 +1320,7 @@ set_promiscuous(scli_interp_t *interp, if_mib_ifEntry_t *ifEntry,
 {
     gint32 *value = (gint32 *) user_data;
 
-    if_mib_proc_set_interface_promiscuous(interp->peer, ifEntry->ifIndex, *value);
+    if_mib_set_ifPromiscuousMode(interp->peer, ifEntry->ifIndex, *value);
     if (interp->peer->error_status) {
 	interface_snmp_error(interp, ifEntry);
     }
@@ -1202,6 +1342,7 @@ set_interface_promiscuous(scli_interp_t *interp, int argc, char **argv)
 
     if (! gsnmp_enum_get_number(if_mib_enums_ifPromiscuousMode,
 				argv[2], &value)) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1217,7 +1358,7 @@ set_notifications(scli_interp_t *interp, if_mib_ifEntry_t *ifEntry,
 {
     gint32 *value = (gint32 *) user_data;
 
-    if_mib_proc_set_notifications(interp->peer, ifEntry->ifIndex, *value);
+    if_mib_set_ifLinkUpDownTrapEnable(interp->peer, ifEntry->ifIndex, *value);
     if (interp->peer->error_status) {
 	interface_snmp_error(interp, ifEntry);
     }
@@ -1239,6 +1380,7 @@ set_interface_notifications(scli_interp_t *interp, int argc, char **argv)
 
     if (! gsnmp_enum_get_number(if_mib_enums_ifLinkUpDownTrapEnable,
 				argv[2], &value)) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1266,7 +1408,8 @@ dump_interface(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_OK;
     }
 
-    if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFENTRY_CONFIG);
+    if_mib_proc_get_ifTable(interp->peer, &ifTable,
+			    IF_MIB_IFENTRY_CONFIG, interp->epoch);
     if (interp->peer->error_status) {
 	return SCLI_SNMP;
     }
@@ -1310,7 +1453,7 @@ dump_interface(scli_interp_t *interp, int argc, char **argv)
 	}
     }
 
-    if (ifTable) if_mib_free_ifTable(ifTable);
+    if (ifTable) if_mib_proc_free_ifTable(ifTable);
     if (ifXTable) if_mib_free_ifXTable(ifXTable);
 
     return SCLI_OK;
@@ -1335,6 +1478,7 @@ alert_interface_status(scli_interp_t *interp, int argc, char **argv)
 
     regex_status = &_regex_status;
     if (regcomp(regex_status, argv[1], interp->regex_flags) != 0) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_REGEXP;
     }
 
@@ -1342,6 +1486,7 @@ alert_interface_status(scli_interp_t *interp, int argc, char **argv)
 	regex_iface = &_regex_iface;
 	if (regcomp(regex_iface, argv[2], interp->regex_flags) != 0) {
 	    if (regex_status) regfree(regex_status);
+	    g_string_assign(interp->result, argv[2]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -1359,7 +1504,7 @@ alert_interface_status(scli_interp_t *interp, int argc, char **argv)
 
     if (ifTable) {
 	for (i = 0; ifTable[i]; i++) {
-	    if (match_interface(regex_iface, ifTable[i])) {
+	    if (interface_match(regex_iface, ifTable[i])) {
 		e = fmt_enum(if_mib_enums_ifOperStatus,
 			     ifTable[i]->ifOperStatus);
 		if (e) {
@@ -1388,54 +1533,86 @@ alert_interface_status(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static int
+create_interface_stack(scli_interp_t *interp, int argc, char **argv)
+{
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    return SCLI_OK;
+}
+
+
+
+static int
+delete_interface_stack(scli_interp_t *interp, int argc, char **argv)
+{
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    return SCLI_OK;
+}
+
+
+
 void
 scli_init_interface_mode(scli_interp_t *interp)
 {
     static scli_cmd_t cmds[] = {
 
+	{ "create interface stack", "<lower-regexp> <higher-regexp>",
+	  "",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
+	  NULL, NULL,
+	  create_interface_stack },
+
+	{ "delete interface stack", "<lower-regexp> <higher-regexp>",
+	  "",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
+	  NULL, NULL,
+	  delete_interface_stack },
+
 	{ "set interface status", "<regexp> <status>",
-	  "The set interface status command modifies the administrative\n"
+	  "The `set interface status' command modifies the administrative\n"
 	  "status of all selected interfaces. The regular expression\n"
 	  "<regexp> is matched against the interface descriptions to\n"
 	  "select the interfaces of interest. The <value> parameter must\n"
 	  "be one of the strings \"up\", \"down\", or \"testing\".",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_interface_status },
 
 	{ "set interface alias", "<regexp> <string>",
-	  "The set interface alias command assigns the alias name\n"
+	  "The `set interface alias' command assigns the alias name\n"
 	  "<string> to the selected interfaces. The alias name provies\n"
-	  "a non-volatile 'handle' which can be used by management\n"
+	  "a non-volatile handle which can be used by management\n"
 	  "applications to better identify interfaces. The regular\n"
 	  "expression <regexp> is matched against the interface\n"
 	  "descriptions to select the interfaces.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_interface_alias },
 
 	{ "set interface notifications", "<regexp> <value>",
-	  "The set interface notifications command controls whether the\n"
+	  "The `set interface notifications' command controls whether the\n"
 	  "selected interfaces generate linkUp and linkDown notifications.\n"
 	  "The regular expression <regexp> is matched against the interface\n"
 	  "descriptions to select the interfaces. The <value> parameter\n"
 	  "must be one of the strings \"enabled\" or \"disabled\".",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_interface_notifications },
 
 	{ "set interface promiscuous", "<regexp> <bool>",
-	  "The set interface promiscuous command controls whether the\n"
+	  "The `set interface promiscuous' command controls whether the\n"
 	  "selected interfaces operate in promiscuous mode or not. The\n"
 	  "regular expression <regexp> is matched against the interface\n"
 	  "descriptions to select the interfaces. The <bool> parameter\n"
 	  "must be one of the strings \"true\" or \"false\".",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_interface_promiscuous },
 
 	{ "show interface info", "[<regexp>]",
-	  "The show interface info command displays summary information\n"
+	  "The `show interface info' command displays summary information\n"
 	  "for all selected interfaces. The optional regular expression\n"
 	  "<regexp> is matched against the interface descriptions to\n"
 	  "select the interfaces of interest. The command generates a\n"
@@ -1462,7 +1639,7 @@ scli_init_interface_mode(scli_interp_t *interp)
 	  show_interface_info },
 	
 	{ "show interface details", "[<regexp>]",
-	  "The show interface details command describes the selected\n"
+	  "The `show interface details' command describes the selected\n"
 	  "interfaces in more detail. The optional regular expression\n"
 	  "<regexp> is matched against the interface descriptions to\n"
 	  "select the interfaces of interest.",
@@ -1471,14 +1648,20 @@ scli_init_interface_mode(scli_interp_t *interp)
 	  show_interface_details },
 	
 	{ "show interface stack", "[<regexp>]",
-	  "The show interface stack command shows the stacking order\n"
-	  "of the interfaces.",
+	  "The `show interface stack' command shows the stacking order\n"
+	  "of the interfaces. The command generates a table with the\n"
+	  "following columns:\n"
+	  "\n"
+	  "  INTERFACE   network interface number\n"
+	  "  STACK       indication of the stacking order\n"
+	  "  TYPE        type of the network interface\n"
+	  "  DESCRIPTION description of the network interface",
 	  SCLI_CMD_FLAG_NEED_PEER,
 	  NULL, NULL,
 	  show_interface_stack },
 	
 	{ "show interface stats", "[<regexp>]",
-	  "The show interface stats command displays network\n"
+	  "The `show interface stats' command displays network\n"
 	  "interface statistics for all selected interfaces.\n"
 	  "The optional regular expression <regexp> is matched\n"
 	  "against the interface description to select the\n"
@@ -1497,9 +1680,9 @@ scli_init_interface_mode(scli_interp_t *interp)
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_interface_stats },
-	
+
 	{ "monitor interface stats", "[<regexp>]",
-	  "The monitor interface stats command shows the same\n"
+	  "The `monitor interface stats' command shows the same\n"
 	  "information as the show interface stats command. The\n"
 	  "information is updated periodically.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR | SCLI_CMD_FLAG_DRY,
@@ -1507,14 +1690,22 @@ scli_init_interface_mode(scli_interp_t *interp)
 	  show_interface_stats },
 
 	{ "alert interface status", "<regexp> [<regexp>]",
-	  "The alarm interface status command generates alerts for\n"
+	  "The `alarm interface status' command generates alerts for\n"
 	  "interfaces that are in given status.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_LOOP,
 	  NULL, NULL,
 	  alert_interface_status },
-	
+#if 0
+	{ "rrd interface stats", "<path> [<regexp>]",
+	  "The `rrd interface stats' command periodically collects\n"
+	  "interface statistics and stores them in round robin\n"
+	  "databases (rrds).",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  rrd_interface_stats },
+#endif
 	{ "dump interface", NULL,
-	  "The dump interface command generates a sequence of scli commands\n"
+	  "The `dump interface' command generates a sequence of scli commands\n"
 	  "which can be used to restore the interface configuration.\n",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
