@@ -1,7 +1,7 @@
 /* 
  * ip.c -- scli ip mode implementation
  *
- * Copyright (C) 2001 Juergen Schoenwaelder
+ * Copyright (C) 2001-2002 Juergen Schoenwaelder
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: ip.c,v 1.35 2001/12/17 18:06:44 schoenw Exp $
+ * @(#) $Id: ip.c,v 1.43 2002/09/02 09:21:26 schoenw Exp $
  */
 
 #include "scli.h"
@@ -28,6 +28,10 @@
 #include "if-mib.h"
 #include "rfc1213-mib.h"
 
+#define RFC1213_MIB_IPROUTE_MASK \
+	( RFC1213_MIB_IPROUTEDEST | RFC1213_MIB_IPROUTEIFINDEX \
+	  | RFC1213_MIB_IPROUTENEXTHOP | RFC1213_MIB_IPROUTETYPE \
+	  | RFC1213_MIB_IPROUTEPROTO | RFC1213_MIB_IPROUTEMASK )
 
 
 GSnmpEnum const forwarding[] = {
@@ -40,9 +44,65 @@ GSnmpEnum const forwarding[] = {
 
 static void
 fmt_ip_forward(GString *s,
-	       ip_forward_mib_ipCidrRouteEntry_t *ipCidrRouteEntry)
+	       ip_forward_mib_ipCidrRouteEntry_t *ipCidrRouteEntry,
+	       if_mib_ifXEntry_t **ifXTable,
+	       if_mib_ifEntry_t **ifTable)
 {
-    g_string_append(s, "\n");
+    const char *label;
+    int i, pos;
+    
+    if (ipCidrRouteEntry->ipCidrRouteIfIndex) {
+	g_string_append(s, "\n");
+	g_string_sprintfa(s, "%s/%s%n",
+			  fmt_ipv4_address(ipCidrRouteEntry->ipCidrRouteDest,
+					   SCLI_FMT_ADDR),
+			  fmt_ipv4_mask(ipCidrRouteEntry->ipCidrRouteMask), &pos);
+	g_string_sprintfa(s, "%*s", MAX(20-pos, 1), "");
+	
+	g_string_sprintfa(s, "  %2d ", ipCidrRouteEntry->ipCidrRouteTos);
+	
+	g_string_sprintfa(s, "%-16s",
+			  fmt_ipv4_address(ipCidrRouteEntry->ipCidrRouteNextHop,
+					   SCLI_FMT_ADDR));
+
+	label = NULL;
+	if (ipCidrRouteEntry->ipCidrRouteType) {
+	    label = gsnmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteType,
+					 *ipCidrRouteEntry->ipCidrRouteType);
+	}
+	g_string_sprintfa(s, "%-10s", label ? label : "");
+
+	label = NULL;
+	if (ipCidrRouteEntry->ipCidrRouteProto) {
+	    label = gsnmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteProto,
+					*ipCidrRouteEntry->ipCidrRouteProto);
+	}
+	g_string_sprintfa(s, "%-10s", label ? label : "");
+
+	g_string_sprintfa(s, "%2d", *ipCidrRouteEntry->ipCidrRouteIfIndex);
+
+	if (ifXTable) {
+	    for (i = 0; ifXTable[i]; i++) {
+		if (ifXTable[i]->ifIndex == *ipCidrRouteEntry->ipCidrRouteIfIndex
+		    && ifXTable[i]->ifName) {
+		    g_string_sprintfa(s, " (%.*s)",
+				      (int) ifXTable[i]->_ifNameLength,
+				      ifXTable[i]->ifName);
+		    break;
+		}
+	    }
+	} else if (ifTable) {
+	    for (i = 0; ifTable[i]; i++) {
+		if (ifTable[i]->ifIndex == *ipCidrRouteEntry->ipCidrRouteIfIndex
+		    && ifTable[i]->ifDescr) {
+		    g_string_sprintfa(s, " (%.*s)",
+				      (int) ifTable[i]->_ifDescrLength,
+				      ifTable[i]->ifDescr);
+		    break;
+		}
+	    }
+	}
+    }
 }
 
 
@@ -65,6 +125,8 @@ fmt_ip_route(GString *s,
 					   SCLI_FMT_ADDR),
 			  fmt_ipv4_mask(ipRouteEntry->ipRouteMask), &pos);
 	g_string_sprintfa(s, "%*s", MAX(20-pos, 1), "");
+
+	g_string_sprintfa(s, "   - ");
 	
 	g_string_sprintfa(s, "%-16s",
 			  fmt_ipv4_address(ipRouteEntry->ipRouteNextHop,
@@ -88,7 +150,8 @@ fmt_ip_route(GString *s,
 
 	if (ifXTable) {
 	    for (i = 0; ifXTable[i]; i++) {
-		if(ifXTable[i]->ifIndex == *ipRouteEntry->ipRouteIfIndex) {
+		if (ifXTable[i]->ifIndex == *ipRouteEntry->ipRouteIfIndex
+		    && ifXTable[i]->ifName) {
 		    g_string_sprintfa(s, " (%.*s)",
 				      (int) ifXTable[i]->_ifNameLength,
 				      ifXTable[i]->ifName);
@@ -97,7 +160,8 @@ fmt_ip_route(GString *s,
 	    }
 	} else if (ifTable) {
 	    for (i = 0; ifTable[i]; i++) {
-		if(ifTable[i]->ifIndex == *ipRouteEntry->ipRouteIfIndex) {
+		if (ifTable[i]->ifIndex == *ipRouteEntry->ipRouteIfIndex
+		    && ifTable[i]->ifDescr) {
 		    g_string_sprintfa(s, " (%.*s)",
 				      (int) ifTable[i]->_ifDescrLength,
 				      ifTable[i]->ifDescr);
@@ -132,7 +196,7 @@ show_ip_forwarding(scli_interp_t *interp, int argc, char **argv)
 
     ip_forward_mib_get_ipCidrRouteTable(interp->peer, &ipCidrRouteTable, 0);
     if (interp->peer->error_status || !ipCidrRouteTable) {
-	rfc1213_mib_get_ipRouteTable(interp->peer, &ipRouteTable, 0);
+	rfc1213_mib_get_ipRouteTable(interp->peer, &ipRouteTable, RFC1213_MIB_IPROUTE_MASK);
 	if (interp->peer->error_status) {
 	    return SCLI_SNMP;
 	}
@@ -141,14 +205,14 @@ show_ip_forwarding(scli_interp_t *interp, int argc, char **argv)
     if (ipCidrRouteTable || ipRouteTable) {
 	if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFDESCR);
 	if_mib_get_ifXTable(interp->peer, &ifXTable, IF_MIB_IFNAME);
-	g_string_sprintfa(interp->header, "%-20s%-16s%-10s%-10s%s",
-		  "DESTINATION", "NEXT HOP", "TYPE", "PROTO", "INTERFACE");
+	g_string_sprintfa(interp->header, "%-20s TOS %-16s%-10s%-10s%s",
+		  "DESTINATION", "NEXT-HOP", "TYPE", "PROTO", "INTERFACE");
 	if (ipCidrRouteTable) {
 	    for (i = 0; ipCidrRouteTable[i]; i++) {
-		fmt_ip_forward(interp->result, ipCidrRouteTable[i]);
+		fmt_ip_forward(interp->result, ipCidrRouteTable[i],
+			       ifXTable, ifTable);
 	    }
-	}
-	if (! ipCidrRouteTable && ipRouteTable) {
+	} else if (ipRouteTable) {
 	    for (i = 0; ipRouteTable[i]; i++) {
 		fmt_ip_route(interp->result, ipRouteTable[i],
 			     ifXTable, ifTable);
@@ -197,26 +261,49 @@ xml_ip_address(xmlNodePtr root, ip_mib_ipAddrEntry_t *ipAddrEntry)
 
 
 static void
-fmt_ip_address(GString *s, ip_mib_ipAddrEntry_t *ipAddrEntry)
+fmt_ip_address(GString *s,
+	       ip_mib_ipAddrEntry_t *ipAddrEntry,
+	       int name_width,
+	       if_mib_ifXEntry_t **ifXTable,
+	       if_mib_ifEntry_t **ifTable)
 {
     char *name;
+    int i;
 
-    if (ipAddrEntry->ipAdEntIfIndex) {
-	g_string_sprintfa(s, "%9u ", *(ipAddrEntry->ipAdEntIfIndex));
-    } else {
-	g_string_sprintfa(s, "%6s    ", "");
-    }
-    g_string_sprintfa(s, "%-16s ",
+    g_string_sprintfa(s, "%-17s ",
 	      fmt_ipv4_address(ipAddrEntry->ipAdEntAddr, SCLI_FMT_ADDR));
     if (ipAddrEntry->ipAdEntNetMask) {
-	g_string_sprintfa(s, "/%-6s",
+	g_string_sprintfa(s, "/%-4s",
 			  fmt_ipv4_mask(ipAddrEntry->ipAdEntNetMask));;
     } else {
-	g_string_sprintfa(s, "%-6s", "");
+	g_string_sprintfa(s, "%-5s", "");
     }
     name = fmt_ipv4_address(ipAddrEntry->ipAdEntAddr, SCLI_FMT_NAME);
-    if (name) {
-	g_string_append(s, name);
+    g_string_sprintfa(s, "%-*s ", name_width, name ? name : "");
+    
+    if (ipAddrEntry->ipAdEntIfIndex) {
+	g_string_sprintfa(s, " %u", *(ipAddrEntry->ipAdEntIfIndex));
+	if (ifXTable) {
+	    for (i = 0; ifXTable[i]; i++) {
+		if (ifXTable[i]->ifIndex == *ipAddrEntry->ipAdEntIfIndex
+		    && ifXTable[i]->ifName) {
+		    g_string_sprintfa(s, " (%.*s)",
+				      (int) ifXTable[i]->_ifNameLength,
+				      ifXTable[i]->ifName);
+		    break;
+		}
+	    }
+	} else if (ifTable) {
+	    for (i = 0; ifTable[i]; i++) {
+		if (ifTable[i]->ifIndex == *ipAddrEntry->ipAdEntIfIndex
+		    && ifTable[i]->ifDescr) {
+		    g_string_sprintfa(s, " (%.*s)",
+				      (int) ifTable[i]->_ifDescrLength,
+				      ifTable[i]->ifDescr);
+		    break;
+		}
+	    }
+	}
     }
     g_string_append(s, "\n");
 }
@@ -227,7 +314,9 @@ static int
 show_ip_addresses(scli_interp_t *interp, int argc, char **argv)
 {
     ip_mib_ipAddrEntry_t **ipAddrTable = NULL;
-    int i;
+    if_mib_ifEntry_t **ifTable = NULL;
+    if_mib_ifXEntry_t **ifXTable = NULL;
+    int i, name_width = 4;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -245,20 +334,36 @@ show_ip_addresses(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (ipAddrTable) {
+	if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFDESCR);
+	if_mib_get_ifXTable(interp->peer, &ifXTable, IF_MIB_IFNAME);
 	if (! scli_interp_xml(interp)) {
+	    for (i = 0; ipAddrTable[i]; i++) {
+		char *name;
+		name = fmt_ipv4_address(ipAddrTable[i]->ipAdEntAddr, SCLI_FMT_NAME);
+		if (name && strlen(name) > name_width) {
+		    name_width = strlen(name);
+		}
+	    }
 	    g_string_sprintfa(interp->header,
-			      "INTERFACE ADDRESS        PREFIX  NAME");
+			      "ADDRESS         PREFIX %-*s INTERFACE",
+			      name_width, "NAME");
 	}
 	for (i = 0; ipAddrTable[i]; i++) {
 	    if (scli_interp_xml(interp)) {
 		xml_ip_address(interp->xml_node, ipAddrTable[i]);
 	    } else {
-		fmt_ip_address(interp->result, ipAddrTable[i]);
+		fmt_ip_address(interp->result, ipAddrTable[i],
+			       name_width, ifXTable, ifTable);
 	    }
 	}
     }
 
-    if (ipAddrTable) ip_mib_free_ipAddrTable(ipAddrTable);
+    if (ifTable)
+	if_mib_free_ifTable(ifTable);
+    if (ifXTable)
+	if_mib_free_ifXTable(ifXTable);
+    if (ipAddrTable)
+	ip_mib_free_ipAddrTable(ipAddrTable);
 
     return SCLI_OK;
 }
@@ -561,6 +666,7 @@ set_ip_forwarding(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (! gsnmp_enum_get_number(forwarding, argv[1], &value)) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -597,6 +703,7 @@ set_ip_ttl(scli_interp_t *interp, int argc, char **argv)
 
     value = strtol(argv[1], &end, 0);
     if (*end || value < 1 || value > 255) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_NUMBER;
     }
 
@@ -665,23 +772,23 @@ scli_init_ip_mode(scli_interp_t *interp)
     static scli_cmd_t cmds[] = {
 
 	{ "set ip forwarding", "<value>",
-	  "The set ip forwarding command controls whether the IP protocol\n"
+	  "The `set ip forwarding' command controls whether the IP protocol\n"
 	  "engine forwards IP datagrams or not. The <value> parameter must\n"
 	  "be one of the strings \"enabled\" or \"disabled\".",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_ip_forwarding },
 	
 	{ "set ip ttl", "<number>",
-	  "The set ip ttl command can be used to change the default\n"
+	  "The `set ip ttl' command can be used to change the default\n"
 	  "time to live (TTL) value used by the IP protocol engine. The\n"
 	  "<number> parameter must be a number between 1 and 255 inclusive.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_ip_ttl },
 	
 	{ "show ip info", NULL,
-	  "The show ip info command displays paramters of the IP\n"
+	  "The `show ip info' command displays parameters of the IP\n"
 	  "protocol engine, such as the default TTL or whether the\n"
 	  "node is forwarding IP packets.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
@@ -689,22 +796,23 @@ scli_init_ip_mode(scli_interp_t *interp)
 	  show_ip_info },
 	
 	{ "show ip forwarding", NULL,
-	  "The show ip forwarding command displays the IP forwarding data\n"
+	  "The `show ip forwarding' command displays the IP forwarding data\n"
 	  "base. The command generates a table with the following columns:\n"
 	  "\n"
-	  "  DESTINATION\n"
-	  "  NEXT HOP\n"
-	  "  TYPE\n"
-	  "  PROTO\n"
-	  "  INTERFACE",
+	  "  DESTINATION destination address and prefix\n"
+	  "  NEXT-HOP    next hop towards the destination\n"
+	  "  TOS         type of service selector\n"
+	  "  TYPE        type (direct/indirect) of the entry\n"
+	  "  PROTO       protocol which created the entry\n"
+	  "  INTERFACE   interface used for forwarding",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_ip_forwarding },
 	
 	{ "show ip addresses", NULL,
-	  "The show ip addresses command displays the IP addresses assigned\n"
-	  "to network interfaces. The command generates a table with the\n"
-	  "following columns:\n"
+	  "The `show ip addresses' command displays the IP addresses\n"
+	  "assigned to network interfaces. The command generates a table\n"
+	  "with the following columns:\n"
 	  "\n"
 	  "  INTERFACE network interface number\n"
 	  "  ADDRESS   IP address\n"
@@ -715,14 +823,14 @@ scli_init_ip_mode(scli_interp_t *interp)
 	  show_ip_addresses },
 	
 	{ "show ip tunnel", NULL,
-	  "The show ip tunnel command displays information about existing\n"
+	  "The `show ip tunnel' command displays information about existing\n"
 	  "IP tunnels.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_ip_tunnel },
 	
 	{ "show ip mapping", NULL,
-	  "The show ip mapping command displays the mapping of IP address\n"
+	  "The `show ip mapping' command displays the mapping of IP address\n"
 	  "to lower layer address (e.g., IEEE 802 addresses). The command\n"
 	  "generates a table with the following columns:\n"
 	  "\n"
@@ -735,7 +843,7 @@ scli_init_ip_mode(scli_interp_t *interp)
 	  show_ip_mapping },
 	
 	{ "dump ip", NULL,
-	  "The dump ip command generates a sequence of scli commands\n"
+	  "The `dump ip' command generates a sequence of scli commands\n"
 	  "which can be used to restore the IP configuration.\n",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
