@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: nortel.c,v 1.26 2002/01/25 09:32:16 schoenw Exp $
+ * @(#) $Id: nortel.c,v 1.34 2002/06/26 17:40:07 schoenw Exp $
  */
 
 #include "scli.h"
@@ -174,73 +174,6 @@ xml_port_set(xmlNodePtr root, guchar *bits, gsize bits_len)
 
 
 
-static void
-fmt_port_set(GString *s, guchar *bits, gsize bits_len)
-{
-    int bit, i;
-    int first = -1;
-    int n = 0;
-
-    for (i = 0; i < bits_len * 8; i++) {
-	bit = bits[i/8] & 1 << (7-(i%8));
-	if (bit) {
-	    if (first < 0) {
-		first = i;
-	    }
-	} else {
-	    if (first >= 0) {
-		if (n) {
-		    g_string_append_c(s, ',');
-		}
-		if (first == i-1) {
-		    g_string_sprintfa(s, "%d", first);
-		} else {
-		    g_string_sprintfa(s, "%d-%d", first, i-1);
-		}
-		n++;
-	    }
-	    first = -1;
-	}
-    }
-    if (first >= 0) {
-	g_string_sprintfa(s, "%s%d-%d", n ? ", " : "", first, i-1);
-    }
-}
-
-
-
-static int
-scan_port_set(guchar *bits, gsize bits_len, char *string)
-{
-    char *p = string;
-    int i, from, to, off;
-
-    memset(bits, 0, bits_len);
-    while (*p) {
-	if (sscanf(p, "%d-%d%n", &from, &to, &off) == 2) {
-	    p += off;
-	} else if (sscanf(p, "%d%n", &from, &off) == 1) {
-	    p += off;
-	    to = from;
-	} else {
-	    memset(bits, 0, bits_len);
-	    return SCLI_ERROR;
-	}
-	if (from < 0 || to >= bits_len*8) {
-	    memset(bits, 0, bits_len);
-	    return SCLI_ERROR;
-	}
-	for (i = from; i < to+1; i++) {
-	    bits[i/8] |= (1 <<(7-(i%8)));
-	}
-	if (*p == ',') p++;
-    }
-
-    return SCLI_OK;
-}
-
-
-
 static void
 fmt_id_list(GString *s, guchar *ids, gint32 numids)
 {
@@ -481,6 +414,7 @@ show_nortel_bridge_vlan_details(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_vlan = &_regex_vlan;
 	if (regcomp(regex_vlan, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -574,6 +508,7 @@ show_nortel_bridge_vlan_info(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_vlan = &_regex_vlan;
 	if (regcomp(regex_vlan, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -709,6 +644,48 @@ show_nortel_bridge_vlan_ports(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static int
+create_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
+{
+    gint32 vlanId;
+    char *end, *name;
+    size_t name_len;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc != 3) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    vlanId = strtol(argv[1], &end, 0);
+    if (*end || vlanId < 0) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_NUMBER;
+    }
+
+    name = argv[2];
+    name_len = strlen(name);
+    if (name_len < RAPID_CITY_RCVLANNAMEMINLENGTH
+	|| name_len > RAPID_CITY_RCVLANNAMEMAXLENGTH) {
+	g_string_assign(interp->result, name);
+	return SCLI_SYNTAX_VALUE;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    rapid_city_proc_create_vlan(interp->peer, vlanId, name, name_len,
+				RAPID_CITY_RCVLANTYPE_BYPORT);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    return SCLI_OK;
+}
+
+
+
 static int
 delete_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
 {
@@ -724,6 +701,7 @@ delete_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
 
     regex_vlan = &_regex_vlan;
     if (regcomp(regex_vlan, argv[1], interp->regex_flags) != 0) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_REGEXP;
     }
 
@@ -732,7 +710,8 @@ delete_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_OK;
     }
 
-    rapid_city_get_rcVlanTable(interp->peer, &vlanTable, RAPID_CITY_RCVLANNAME);
+    rapid_city_get_rcVlanTable(interp->peer, &vlanTable,
+			       RAPID_CITY_RCVLANNAME);
     if (interp->peer->error_status) {
 	regfree(regex_vlan);
 	return SCLI_SNMP;
@@ -759,10 +738,12 @@ delete_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
 
 
 static int
-create_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
+set_nortel_bridge_vlan_name(scli_interp_t *interp, int argc, char **argv)
 {
+    rapid_city_rcVlanEntry_t *vlanEntry = NULL;
     gint32 vlanId;
-    char *end;
+    char *end, *name;
+    size_t name_len;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -771,21 +752,42 @@ create_nortel_bridge_vlan(scli_interp_t *interp, int argc, char **argv)
     }
 
     vlanId = strtol(argv[1], &end, 0);
-    if (*end) {
+    if (*end || vlanId < 0) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_NUMBER;
     }
 
+    name = argv[2];
+    name_len = strlen(name);
+    if (name_len < RAPID_CITY_RCVLANNAMEMINLENGTH
+	|| name_len > RAPID_CITY_RCVLANNAMEMAXLENGTH) {
+	g_string_assign(interp->result, name);
+	return SCLI_SYNTAX_VALUE;
+    }
+
     if (scli_interp_dry(interp)) {
 	return SCLI_OK;
     }
 
-    rapid_city_proc_create_vlan(interp->peer, vlanId, argv[2],
-				RAPID_CITY_RCVLANTYPE_BYPORT);
+    rapid_city_get_rcVlanEntry(interp->peer, &vlanEntry, vlanId,
+			       RAPID_CITY_RCVLANNAME);
     if (interp->peer->error_status) {
-	vlan_snmp_error(interp, NULL);
-	return SCLI_ERROR;
+	return SCLI_SNMP;
     }
 
+    if (vlanEntry) {
+	vlanEntry->rcVlanName = name;
+	vlanEntry->_rcVlanNameLength = name_len;
+	rapid_city_set_rcVlanEntry(interp->peer, vlanEntry,
+				   RAPID_CITY_RCVLANNAME);
+	if (interp->peer->error_status) {
+	    rapid_city_free_rcVlanEntry(vlanEntry);
+	    return SCLI_SNMP;
+	}
+    }
+
+    if (vlanEntry) rapid_city_free_rcVlanEntry(vlanEntry);
+
     return SCLI_OK;
 }
 
@@ -807,11 +809,13 @@ set_nortel_bridge_vlan_ports(scli_interp_t *interp, int argc, char **argv)
 
     regex_vlan = &_regex_vlan;
     if (regcomp(regex_vlan, argv[1], interp->regex_flags) != 0) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_REGEXP;
     }
 
     if (scan_port_set(ports, sizeof(ports), argv[2]) != SCLI_OK) {
 	if (regex_vlan) regfree(regex_vlan);
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -862,6 +866,7 @@ set_nortel_bridge_vlan_default(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (scan_port_set(ports, sizeof(ports), argv[2]) != SCLI_OK) {
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -970,66 +975,74 @@ scli_init_nortel_mode(scli_interp_t *interp)
 {
     static scli_cmd_t cmds[] = {
 	{ "create nortel bridge vlan", "<vlanid> <name>",
-	  "The create nortel bridge vlan command is used to create a\n"
-	  "new vlan with the given <vlanid> and <name>.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  "The `create nortel bridge vlan' command is used to create a\n"
+	  "new virtual LAN with the given <vlanid> and <name>.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  create_nortel_bridge_vlan },
 
 	{ "delete nortel bridge vlan", "<regexp>",
-	  "The delete nortel bridge vlan command deletes all selected\n"
-	  "vlans. The regular expression <regexp> is matched against the\n"
-	  "vlan names to select the vlans that should be deleted.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  "The `delete nortel bridge vlan' command deletes all selected\n"
+	  "virtual LANs. The regular expression <regexp> is matched\n"
+	  "against the virtual LAN names to select the vlans that should\n"
+	  "be deleted.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  delete_nortel_bridge_vlan },
 
+	{ "set nortel bridge vlan name", "<vlanid> <name>",
+	  "The `set nortel bridge vlan name' command changes the name of\n"
+	  "a virtual LAN.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
+	  NULL, NULL,
+	  set_nortel_bridge_vlan_name },
+
 	{ "set nortel bridge vlan ports", "<regexp> <ports>",
-	  "The set nortel bridge vlan ports command allows to assign\n"
+	  "The `set nortel bridge vlan ports' command allows to assign\n"
 	  "ports to port-based vlans. The regular expression <regexp>\n"
 	  "is matched against the vlan names to select the vlans that\n"
 	  "should be modified. The <ports> argument contains a comma\n"
 	  "separated list of port numbers or port number ranges, e.g.\n"
 	  "1,5,7-8.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_nortel_bridge_vlan_ports },
 
 	{ "set nortel bridge vlan default", "<string> <ports>",
-	  "The set nortel bridge vlan default command allows to assign\n"
+	  "The `set nortel bridge vlan default' command allows to assign\n"
 	  "ports to a default vlan. The <string> argument is matched\n"
 	  "against the vlan names to select the vlan. The <ports> argument\n"
 	  "contains a comma separated list of port numbers or port number\n"
 	  "ranges, e.g. 1,5,7-8.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_nortel_bridge_vlan_default },
 
 	{ "show nortel bridge vlan info", "[<regexp>]",
-	  "The show nortel bridge vlan info command shows summary\n"
-	  "information about all selected vlans. The optional regular\n"
-	  "expression <regexp> is matched against the vlan names to\n"
-	  "select the vlans of interest. The command generates a table\n"
-	  "with the following columns:\n"
+	  "The `show nortel bridge vlan info' command shows summary\n"
+	  "information about all selected virtual LANs. The optional\n"
+	  "regular expression <regexp> is matched against the virtual\n"
+	  "LAN names to select the virtual LANs of interest. The\n"
+	  "command generates a table with the following columns:\n"
 	  "\n"
-	  "  VLAN   vlan number\n"
-	  "  STATUS status of the vlan (see below)\n"
-	  "  NAME   vlan name\n"
-	  "  PORTS  ports assigned to the vlan\n"
+	  "  VLAN   number of the virtual LAN\n"
+	  "  STATUS status of the virtual LAN (see below)\n"
+	  "  NAME   name of the virtual LAN\n"
+	  "  PORTS  ports assigned to the virtual LAN\n"
 	  "\n"
 	  "The status is encoded in four characters. The first character\n"
 	  "indicates the status of the row (A=active, S=not in service,\n"
-	  "R=not ready). The second character indicates  vlan type (P=B\n"
-	  "port, I=IP-subnet, O=protocol, S=src address, D=dst address).\n"
-	  "The third character indicates the priority of the vlan (H=high,\n"
-	  "N=normal) and the fourth character indicates whether routing\n"
-	  "is enabled (R=routing, N=no routing).",
+	  "R=not ready). The second character indicates virtual LAN type\n"
+	  "(P=port, I=IP-subnet, O=protocol, S=src address, D=dst address).\n"
+	  "The third character indicates the priority of the virtual LAN\n"
+	  "(H=high, N=normal) and the fourth character indicates whether\n"
+	  "routing is enabled (R=routing, N=no routing).",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_nortel_bridge_vlan_info },
 
 	{ "show nortel bridge vlan details", "[<regexp>]",
-	  "The show nortel bridge vlan details command describes the\n"
+	  "The `show nortel bridge vlan details' command describes the\n"
 	  "selected vlans in more detail. The optional regular expression\n"
 	  "<regexp> is matched against the vlan names to select the vlans\n"
 	  "of interest.",
@@ -1038,7 +1051,7 @@ scli_init_nortel_mode(scli_interp_t *interp)
 	  show_nortel_bridge_vlan_details },
 
 	{ "show nortel bridge vlan ports", NULL,
-	  "The show nortel bridge vlan ports command shows information\n"
+	  "The `show nortel bridge vlan ports' command shows information\n"
 	  "for each vlan port. The command generates a table with the\n"
 	  "following columns:\n"
 	  "\n"
@@ -1058,8 +1071,8 @@ scli_init_nortel_mode(scli_interp_t *interp)
 	  show_nortel_bridge_vlan_ports },
 
 	{ "dump nortel bridge vlan", NULL,
-	  "The dump nortel bridge vlan command generates a sequence of scli\n"
-	  "commands which can be used to restore the vlan configuration.\n",
+	  "The `dump nortel bridge vlan' command generates a sequence of scli\n"
+	  "commands which can be used to restore the virtual LAN configuration.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  dump_nortel_bridge_vlan },
@@ -1069,7 +1082,7 @@ scli_init_nortel_mode(scli_interp_t *interp)
     
     static scli_mode_t nortel_mode = {
 	"nortel",
-	"The nortel scli mode allows to manipulate virtual lans (vlans)\n"
+	"The nortel scli mode allows to manipulate virtual LANs (vlans)\n"
 	"on nortel bridges. It is based on the RAPID-CITY MIB which\n"
 	"is implemented at least on the baystack bridges.",
 	cmds
