@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp.c,v 1.26 2002/03/26 16:52:16 schoenw Exp $
+ * @(#) $Id: snmp.c,v 1.31 2002/07/19 09:17:16 schoenw Exp $
  */
 
 #include "scli.h"
@@ -30,6 +30,7 @@
 #include "snmp-notification-mib.h"
 
 #include "snmp-view-based-acm-mib-proc.h"
+#include "snmp-user-based-sm-mib-proc.h"
 
 
 static GSnmpEnum const mp_model[] = {
@@ -110,62 +111,6 @@ creatable(gint32 *storage)
 
 
 
-static void
-fmt_storage_type(GString *s, gint32 *storage)
-{
-    static GSnmpEnum const storage_types[] = {
-	{ 1, "O" },	/* other */
-	{ 2, "V" },	/* volatile */
-	{ 3, "N" },	/* nonVolatile */
-	{ 4, "P" },	/* permanent */
-	{ 5, "R" },	/* readOnly */
-	{ 0, NULL }
-    };
-    
-    const char *label;
-    
-    if (! storage) {
-	g_string_append(s, " ");
-	return;
-    }
-    
-    label = gsnmp_enum_get_label(storage_types, *storage);
-    if (label) {
-	g_string_append(s, label);
-    } else {
-	g_string_append(s, "-");
-    }
-}
-
-
-
-static void
-fmt_row_status(GString *s, gint32 *status)
-{
-    static GSnmpEnum const row_states[] = {
-	{ 1, "A" },	/* active */
-	{ 2, "S" },	/* notInService */
-	{ 3, "R" },	/* notReady */
-	{ 0, NULL }
-    };
-    
-    const char *label;
-    
-    if (! status) {
-	g_string_append(s, " ");
-	return;
-    }
-    
-    label = gsnmp_enum_get_label(row_states, *status);
-    if (label) {
-	g_string_append(s, label);
-    } else {
-	g_string_append(s, "-");
-    }
-}
-
-
-
 static int
 create_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
 {
@@ -177,16 +122,21 @@ create_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (strlen(argv[1]) < 1 || strlen(argv[1]) > 32) {
+    if (strlen(argv[1]) < SNMP_VIEW_BASED_ACM_MIB_VACMSECURITYNAMEMINLENGTH
+	|| strlen(argv[1]) > SNMP_VIEW_BASED_ACM_MIB_VACMSECURITYNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
 
-    if (strlen(argv[2]) < 1 || strlen(argv[2]) > 32) {
+    if (strlen(argv[2]) < SNMP_VIEW_BASED_ACM_MIB_VACMGROUPNAMEMINLENGTH
+	|| strlen(argv[2]) > SNMP_VIEW_BASED_ACM_MIB_VACMGROUPNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_VALUE;
     }
 
     if (argc == 4) {
 	if (! gsnmp_enum_get_number(security_model, argv[3], &model)) {
+	    g_string_assign(interp->result, argv[3]);
 	    return SCLI_SYNTAX_VALUE;
 	}
     }
@@ -224,12 +174,14 @@ delete_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
 
     regex_name = &_regex_name;
     if (regcomp(regex_name, argv[1], interp->regex_flags) != 0) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_REGEXP;
     }
 
     regex_group = &_regex_group;
     if (regcomp(regex_group, argv[2], interp->regex_flags) != 0) {
 	if (regex_name) regfree(regex_name);
+	g_string_assign(interp->result, argv[2]);
 	return SCLI_SYNTAX_REGEXP;
     }
 
@@ -237,6 +189,7 @@ delete_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
 	if (! gsnmp_enum_get_number(security_model, argv[3], &model)) {
 	    if (regex_name) regfree(regex_name);
 	    if (regex_group) regfree(regex_group);
+	    g_string_assign(interp->result, argv[3]);
 	    return SCLI_SYNTAX_VALUE;
 	}
     }
@@ -298,6 +251,43 @@ delete_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static int
+create_snmp_usm_user(scli_interp_t *interp, int argc, char **argv)
+{
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc != 3) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (strlen(argv[1]) < SNMP_USER_BASED_SM_MIB_USMUSERNAMEMINLENGTH
+	|| strlen(argv[1]) > SNMP_USER_BASED_SM_MIB_USMUSERNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_VALUE;
+    }
+
+    if (strlen(argv[2]) < SNMP_USER_BASED_SM_MIB_USMUSERNAMEMINLENGTH
+	|| strlen(argv[2]) > SNMP_USER_BASED_SM_MIB_USMUSERNAMEMAXLENGTH) {
+	g_string_assign(interp->result, argv[2]);
+	return SCLI_SYNTAX_VALUE;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+    
+    snmp_user_based_sm_mib_proc_clone_user(interp->peer,
+					   argv[1], strlen(argv[1]),
+					   argv[2], strlen(argv[2]));
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+    
+    return SCLI_OK;
+}
+
+
+
 static void
 fmt_snmp_engine(GString *s,
 		snmp_framework_mib_snmpEngine_t *snmpEngine,
@@ -1108,7 +1098,6 @@ show_snmp_notification_targets(scli_interp_t *interp, int argc, char **argv)
 static int
 set_snmp_authentication_traps(scli_interp_t *interp, int argc, char **argv)
 {
-    snmpv2_mib_snmp_t *snmp = NULL;
     gint32 value;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
@@ -1119,6 +1108,7 @@ set_snmp_authentication_traps(scli_interp_t *interp, int argc, char **argv)
 
     if (! gsnmp_enum_get_number(snmpv2_mib_enums_snmpEnableAuthenTraps,
 				argv[1], &value)) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
 
@@ -1126,11 +1116,7 @@ set_snmp_authentication_traps(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_OK;
     }
 
-    snmp = snmpv2_mib_new_snmp();
-    snmp->snmpEnableAuthenTraps = &value;
-    snmpv2_mib_set_snmp(interp->peer, snmp, SNMPV2_MIB_SNMPENABLEAUTHENTRAPS);
-    snmpv2_mib_free_snmp(snmp);
-
+    snmpv2_mib_set_snmpEnableAuthenTraps(interp->peer, value);
     if (interp->peer->error_status) {
 	return SCLI_SNMP;
     }
@@ -1212,32 +1198,39 @@ scli_init_snmp_mode(scli_interp_t *interp)
     static scli_cmd_t cmds[] = {
 
 	{ "create snmp vacm member", "<name> <group> [<model>]",
-	  "The create snmp vacm member commands can be used to assign\n"
+	  "The `create snmp vacm member' commands can be used to assign\n"
 	  "new members (security names) to vacm groups. New groups are\n"
 	  "created if they do not exist.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  create_snmp_vacm_member },
 
 	{ "delete snmp vacm member", "<regex-name> <regex-group> [<model>]",
-	  "The delete snmp vacm member commands can be used to delete\n"
+	  "The `delete snmp vacm member' commands can be used to delete\n"
 	  "members (security names) from vacm groups. Groups are deleted\n"
 	  "if the last member is deleted.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  delete_snmp_vacm_member },
 
+	{ "create snmp usm user", "<name> <template>",
+	  "The `create snmp usm user' commands can be used to create a\n"
+	  "new user by cloning an existing template.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
+	  NULL, NULL,
+	  create_snmp_usm_user },
+
 	{ "set snmp authentication traps", "<status>",
-	  "The set snmp authentication traps command controls whether the\n"
+	  "The `set snmp authentication traps' command controls whether the\n"
 	  "SNMP engine generates authentication failure notifications.\n"
 	  "The <value> parameter must be one of the strings \"enabled\"\n"
 	  "or \"disabled\".",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_snmp_authentication_traps },
 
 	{ "show snmp engine", NULL,
-	  "The show snmp engine command displays information about the\n"
+	  "The `show snmp engine' command displays information about the\n"
 	  "SNMP protocol engine such as the number of boots, the current\n"
 	  "time relative to the last boot and the maximum message size.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
@@ -1245,14 +1238,14 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_engine },
 
 	{ "show snmp resources", NULL,
-	  "The show snmp resources command displays information about the\n"
+	  "The `show snmp resources' command displays information about the\n"
 	  "MIB resources supported by the SNMP agent.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_snmp_resources },
 
 	{ "show snmp vacm member", NULL,
-	  "The show snmp vacm member command displays the mapping of\n"
+	  "The `show snmp vacm member' command displays the mapping of\n"
 	  "security names to group names. The command generates a table\n"
 	  "with the following columns:\n"
 	  "\n"
@@ -1265,7 +1258,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_vacm_member },
 
 	{ "show snmp vacm access", NULL,
-	  "The show snmp vacm access command display the access control\n"
+	  "The `show snmp vacm access' command display the access control\n"
 	  "rules for the security groups. The command generates a table\n"
 	  "with the following columns:\n"
 	  "\n"
@@ -1283,7 +1276,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_vacm_access },
 
 	{ "show snmp vacm views", NULL,
-	  "The show snmp vacm views command displays MIB view definitions.\n"
+	  "The `show snmp vacm views' command displays MIB view definitions.\n"
 	  "The command generates a table with the following columns:\n"
 	  "\n"
 	  "  ROW    row storage type and status\n"
@@ -1295,7 +1288,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_vacm_views },
 
 	{ "show snmp usm users", NULL,
-	  "The show snmp usm users command displays the configured users.\n"
+	  "The `show snmp usm users' command displays the configured users.\n"
 	  "The command generates a table with the following columns:\n"
 	  "\n"
 	  "  ROW    row storage type and status\n"
@@ -1308,7 +1301,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_usm_users },
 
 	{ "show snmp target addresses", NULL,
-	  "The show snmp target addresses command displays information\n"
+	  "The `show snmp target addresses' command displays information\n"
 	  "about the configured SNMP target addresses. The command\n"
 	  "generates a table with the following columns:\n"
 	  "\n"
@@ -1325,7 +1318,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_target_addresses },
 
 	{ "show snmp target parameters", NULL,
-	  "The show snmp target parameters command displays information\n"
+	  "The `show snmp target parameters' command displays information\n"
 	  "about the configured SNMP target parameters. The command\n"
 	  "generates a table with the following columns:\n"
 	  "\n"
@@ -1337,7 +1330,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_target_parameters },
 
 	{ "show snmp notification targets", NULL,
-	  "The show snmp notification targets command displays information\n"
+	  "The `show snmp notification targets' command displays information\n"
 	  "about the configured SNMP notification targets. The command\n"
 	  "generates a table with the following columns:\n"
 	  "\n"
@@ -1350,7 +1343,7 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  show_snmp_notification_targets },
 
 	{ "dump snmp", NULL,
-	  "The dump snmp command generates a sequence of scli commands\n"
+	  "The `dump snmp' command generates a sequence of scli commands\n"
 	  "which can be used to restore the engine configuration.\n",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
