@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: system.c,v 1.66 2002/03/27 13:40:34 schoenw Exp $
+ * @(#) $Id: system.c,v 1.73 2002/07/19 09:17:16 schoenw Exp $
  */
 
 #include "scli.h"
@@ -230,11 +230,11 @@ static GSnmpEnum const hrSWRunStatus[] = {
     { 0, NULL }
 };
 
-static GSnmpEnum const hrSWInstalledType[] = {
-    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_UNKNOWN,		"?" },
-    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_OPERATINGSYSTEM,	"O" },
-    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_DEVICEDRIVER,	"D" },
-    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_APPLICATION,		"A" },
+static GSnmpEnum const hrSWRunType[] = {
+    { HOST_RESOURCES_MIB_HRSWRUNTYPE_UNKNOWN,		"?" },
+    { HOST_RESOURCES_MIB_HRSWRUNTYPE_OPERATINGSYSTEM,	"O" },
+    { HOST_RESOURCES_MIB_HRSWRUNTYPE_DEVICEDRIVER,	"D" },
+    { HOST_RESOURCES_MIB_HRSWRUNTYPE_APPLICATION,	"A" },
     { 0, NULL }
 };
 
@@ -374,7 +374,8 @@ xml_system_device(xmlNodePtr root,
 
 static void
 fmt_system_device(GString *s,
-		  host_resources_mib_hrDeviceEntry_t *hrDeviceEntry)
+		  host_resources_mib_hrDeviceEntry_t *hrDeviceEntry,
+		  int type_width)
 {
     char const *status;
     char const *type;
@@ -388,7 +389,7 @@ fmt_system_device(GString *s,
     type = gsnmp_identity_get_label(host_resources_types_identities,
 				    hrDeviceEntry->hrDeviceType,
 				    hrDeviceEntry->_hrDeviceDescrLength);
-    g_string_sprintfa(s, "%8s", type ? type : "");
+    g_string_sprintfa(s, "%*s ", type_width, type ? type : "");
 	
     if (hrDeviceEntry->hrDeviceDescr) {
 	g_string_sprintfa(s, "%.*s\n",
@@ -403,6 +404,8 @@ static int
 show_system_devices(scli_interp_t *interp, int argc, char **argv)
 {
     host_resources_mib_hrDeviceEntry_t **hrDeviceTable = NULL;
+    int type_width = 8;
+    char const *type;
     int i;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
@@ -422,13 +425,23 @@ show_system_devices(scli_interp_t *interp, int argc, char **argv)
     
     if (hrDeviceTable) {
 	if (! scli_interp_xml(interp)) {
-	    g_string_sprintfa(interp->header, "INDEX STATUS  TYPE    DESCRIPTION");
+	    for (i = 0; hrDeviceTable[i]; i++) {
+		type = gsnmp_identity_get_label(host_resources_types_identities,
+					hrDeviceTable[i]->hrDeviceType,
+					hrDeviceTable[i]->_hrDeviceDescrLength);
+		if (type) {
+		    type_width = MAX(type_width, strlen(type));
+		}
+	    }
+	    g_string_sprintfa(interp->header, "INDEX STATUS  %-*s DESCRIPTION",
+			      type_width, "TYPE");
 	}
 	for (i = 0; hrDeviceTable[i]; i++) {
 	    if (scli_interp_xml(interp)) {
 		xml_system_device(interp->xml_node, hrDeviceTable[i]);
 	    } else {
-		fmt_system_device(interp->result, hrDeviceTable[i]);
+		fmt_system_device(interp->result, hrDeviceTable[i],
+				  type_width);
 	    }
 	}
     }
@@ -500,12 +513,12 @@ fmt_system_process(GString *s,
 {
     const char *e;
     
-    g_string_sprintfa(s, "%5d ", hrSWRunEntry->hrSWRunIndex);
+    g_string_sprintfa(s, "%7d ", hrSWRunEntry->hrSWRunIndex);
 
     e = fmt_enum(hrSWRunStatus, hrSWRunEntry->hrSWRunStatus);
     g_string_sprintfa(s, "%s ", e ? e : " ");
     
-    e = fmt_enum(hrSWInstalledType, hrSWRunEntry->hrSWRunType);
+    e = fmt_enum(hrSWRunType, hrSWRunEntry->hrSWRunType);
     g_string_sprintfa(s, "%s ", e ? e : " ");
 
     if (hrSWRunPerfEntry
@@ -519,7 +532,7 @@ fmt_system_process(GString *s,
 	g_string_sprintfa(s, " %s",
 		  fmt_seconds((guint32) *(hrSWRunPerfEntry->hrSWRunPerfCPU)/100));
     } else {
-	g_string_sprintfa(s, " %5s", "--:--");
+	g_string_sprintfa(s, " %6s", "--:--");
     }
     if (hrSWRunEntry->hrSWRunPath
 	&& hrSWRunEntry->_hrSWRunPathLength) {
@@ -559,6 +572,7 @@ show_system_processes(scli_interp_t *interp, int argc, char **argv)
     if (argc == 2) {
 	regex_path = &_regex_path;
 	if (regcomp(regex_path, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
 	    return SCLI_SYNTAX_REGEXP;
 	}
     }
@@ -578,7 +592,7 @@ show_system_processes(scli_interp_t *interp, int argc, char **argv)
 						&hrSWRunPerfTable, 0);
 	if (! scli_interp_xml(interp)) {
 	    g_string_append(interp->header,
-			    "  PID S T MEMORY     TIME COMMAND");
+			    "    PID S T MEMORY      TIME COMMAND");
 	}
 	for (i = 0; hrSWRunTable[i]; i++) {
 	    if (match_process(regex_path, hrSWRunTable[i])) {
@@ -928,31 +942,24 @@ xml_system_info(xmlNodePtr root, scli_interp_t *interp,
 		snmpv2_mib_system_t *system)
 {
     if (system) {
-	if (system->sysName && system->_sysNameLength) {
-	    (void) xml_new_child(root, NULL, "name", "%.*s",
-				 (int) system->_sysNameLength,
-				 system->sysName);
-	}
+	(void) xml_new_child(root, NULL, "name", "%.*s",
+			     (int) system->_sysNameLength,
+			     system->sysName);
 	if (interp->peer->taddress) {
 	    (void) xml_new_child(root, NULL, "taddress", "%s:%d",
 				 interp->peer->taddress,
 				 interp->peer->port);
 	}
-	if (system->sysDescr && system->_sysDescrLength) {
-	    (void) xml_new_child(root, NULL, "description", "%.*s",
-				 (int) system->_sysDescrLength,
-				 system->sysDescr);
-	}
-	if (system->sysContact && system->_sysContactLength) {
-	    (void) xml_new_child(root, NULL, "contact", "%.*s",
-				 (int) system->_sysContactLength,
-				 system->sysContact);
-	}
-	if (system->sysLocation && system->_sysLocationLength) {
-	    (void) xml_new_child(root, NULL, "location", "%.*s",
-				 (int) system->_sysLocationLength,
-				 system->sysLocation);
-	}
+	
+	(void) xml_new_child(root, NULL, "description", "%.*s",
+			     (int) system->_sysDescrLength,
+			     system->sysDescr);
+	(void) xml_new_child(root, NULL, "contact", "%.*s",
+			     (int) system->_sysContactLength,
+			     system->sysContact);
+	(void) xml_new_child(root, NULL, "location", "%.*s",
+			     (int) system->_sysLocationLength,
+			     system->sysLocation);
 
 	if (system->sysObjectID) {
 	    scli_vendor_t const *vendor;
@@ -1102,18 +1109,31 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
     s = interp->result;
     if (hrSystem) {
 	if (hrSystem->hrSystemDate && hrSystem->_hrSystemDateLength) {
-	    g_string_sprintfa(s, "%-*s", indent, "Current Time:");
-	    g_string_append(s, fmt_date_and_time(hrSystem->hrSystemDate,
-					 hrSystem->_hrSystemDateLength));
-	    g_string_append_c(s, '\n');
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "current-time", "%s",
+			     fmt_date_and_time(hrSystem->hrSystemDate,
+					       hrSystem->_hrSystemDateLength));
+	    } else {
+		g_string_sprintfa(s, "%-*s", indent, "Current Time:");
+		g_string_append(s, fmt_date_and_time(hrSystem->hrSystemDate,
+						     hrSystem->_hrSystemDateLength));
+		g_string_append_c(s, '\n');
+	    }
 	}
     }
 
     if (system) {
 	if (system->sysUpTime) {
-	    g_string_sprintfa(s, "%-*s", indent, "Agent Boot Time:");
-	    g_string_append(s, fmt_timeticks(*(system->sysUpTime)));
-	    g_string_append_c(s, '\n');
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "agent-boot-time", "%s",
+				     fmt_timeticks(*(system->sysUpTime)));
+	    } else {
+		g_string_sprintfa(s, "%-*s", indent, "Agent Boot Time:");
+		g_string_append(s, fmt_timeticks(*(system->sysUpTime)));
+		g_string_append_c(s, '\n');
+	    }
 	}
     }
     
@@ -1165,8 +1185,14 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 
     if (interfaces) {
 	if (interfaces->ifNumber) {
-	    g_string_sprintfa(s, "%-*s%d\n", indent, "Interfaces:",
-			      *(interfaces->ifNumber));
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "interfaces", "%d",
+				     *(interfaces->ifNumber));
+	    } else {
+		g_string_sprintfa(s, "%-*s%d\n", indent, "Interfaces:",
+				  *(interfaces->ifNumber));
+	    }
 	}
 	if (ifMibObjects && system && system->sysUpTime) {
 	    if (ifMibObjects->ifTableLastChange) {
@@ -1229,7 +1255,8 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 static int
 set_system_contact(scli_interp_t *interp, int argc, char **argv)
 {
-    snmpv2_mib_system_t *system = NULL;
+    gchar *contact;
+    gsize contact_len;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -1237,16 +1264,19 @@ set_system_contact(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
+    contact = argv[1];
+    contact_len = strlen(contact);
+    if (contact_len < SNMPV2_MIB_SYSCONTACTMINLENGTH
+	|| contact_len > SNMPV2_MIB_SYSCONTACTMAXLENGTH) {
+	g_string_assign(interp->result, contact);
+	return SCLI_SYNTAX_VALUE;
+    }
+
     if (scli_interp_dry(interp)) {
 	return SCLI_OK;
     }
 
-    system = snmpv2_mib_new_system();
-    system->sysContact = argv[1];
-    system->_sysContactLength = strlen(system->sysContact);
-    snmpv2_mib_set_system(interp->peer, system, SNMPV2_MIB_SYSCONTACT);
-    snmpv2_mib_free_system(system);
-
+    snmpv2_mib_set_sysContact(interp->peer, contact, contact_len);
     if (interp->peer->error_status) {
 	return SCLI_SNMP;
     }
@@ -1259,7 +1289,8 @@ set_system_contact(scli_interp_t *interp, int argc, char **argv)
 static int
 set_system_name(scli_interp_t *interp, int argc, char **argv)
 {
-    snmpv2_mib_system_t *system = NULL;
+    gchar *name;
+    gsize name_len;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -1267,16 +1298,19 @@ set_system_name(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
+    name = argv[1];
+    name_len = strlen(name);
+    if (name_len < SNMPV2_MIB_SYSNAMEMINLENGTH
+	|| name_len > SNMPV2_MIB_SYSNAMEMAXLENGTH) {
+	g_string_assign(interp->result, name);
+	return SCLI_SYNTAX_VALUE;
+    }
+
     if (scli_interp_dry(interp)) {
 	return SCLI_OK;
     }
 
-    system = snmpv2_mib_new_system();
-    system->sysName = argv[1];
-    system->_sysNameLength = strlen(system->sysName);
-    snmpv2_mib_set_system(interp->peer, system, SNMPV2_MIB_SYSNAME);
-    snmpv2_mib_free_system(system);
-    
+    snmpv2_mib_set_sysName(interp->peer, name, name_len);
     if (interp->peer->error_status) {
 	return SCLI_SNMP;
     }
@@ -1289,7 +1323,8 @@ set_system_name(scli_interp_t *interp, int argc, char **argv)
 static int
 set_system_location(scli_interp_t *interp, int argc, char **argv)
 {
-    snmpv2_mib_system_t *system = NULL;
+    gchar *location;
+    gsize location_len;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -1297,16 +1332,19 @@ set_system_location(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
+    location = argv[1];
+    location_len = strlen(location);
+    if (location_len < SNMPV2_MIB_SYSLOCATIONMINLENGTH
+	|| location_len > SNMPV2_MIB_SYSLOCATIONMAXLENGTH) {
+	g_string_assign(interp->result, location);
+	return SCLI_SYNTAX_VALUE;
+    }
+
     if (scli_interp_dry(interp)) {
 	return SCLI_OK;
     }
 
-    system = snmpv2_mib_new_system();
-    system->sysLocation = argv[1];
-    system->_sysLocationLength = strlen(system->sysLocation);
-    snmpv2_mib_set_system(interp->peer, system, SNMPV2_MIB_SYSLOCATION);
-    snmpv2_mib_free_system(system);
-
+    snmpv2_mib_set_sysLocation(interp->peer, location, location_len);
     if (interp->peer->error_status) {
 	return SCLI_SNMP;
     }
@@ -1381,6 +1419,7 @@ cmd_xxx(scli_interp_t *interp, int argc, char **argv)
 
     n = strtol(argv[1], &end, 0);
     if (*end) {
+	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_NUMBER;
     }
 
@@ -1412,36 +1451,36 @@ scli_init_system_mode(scli_interp_t *interp)
     static scli_cmd_t cmds[] = {
 
 	{ "set system contact", "<string>",
-	  "The set system contact command configures the system's contact\n"
+	  "The `set system contact' command configures the system contact\n"
 	  "information. The <string> argument should include information\n"
 	  "on how to contact a person who is responsible for this system.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_system_contact },
 
 	{ "set system name", "<string>",
-	  "The set system name command configures the systems's name. By\n"
-	  "convention, this is the system's fully-qualified domain name.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  "The `set system name' command configures the name of the system.\n"
+	  "By convention, this is the fully-qualified domain name.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_system_name },
 
 	{ "set system location", "<string>",
-	  "The set system location command configures the system's physical\n"
-	  "location.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  "The `set system location' command configures the physical\n"
+	  "location of the system.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY | SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  set_system_location },
 
 	{ "show system info", NULL,
-	  "The show system info command shows general information about the\n"
+	  "The `show system info' command shows general information about the\n"
 	  "system.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_XML | SCLI_CMD_FLAG_DRY,
 	  "system info", NULL,
 	  show_system_info },
 
 	{ "show system devices", NULL,
-	  "The show system devices command shows a list of system devices.\n"
+	  "The `show system devices' command shows a list of system devices.\n"
 	  "The command generates a table with the following columns:\n"
 	  "\n"
 	  "  INDEX       device number\n"
@@ -1452,7 +1491,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_devices },
 
 	{ "show system storage", NULL,
-	  "The show system storage command displays information about the\n"
+	  "The `show system storage' command displays information about the\n"
 	  "logical areas attached in the system. The command generates a\n"
 	  "table with the following columns:\n"
 	  "\n"
@@ -1468,7 +1507,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_storage },
 
 	{ "show system mounts", NULL,
-	  "The show system mounts command shows the list of filesystems\n"
+	  "The `show system mounts' command shows the list of filesystems\n"
 	  "mounted on the system. The command generates a table with the\n"
 	  "following columns:\n"
 	  "\n"
@@ -1482,7 +1521,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_mounts },
 
 	{ "show system processes", "[<regexp>]",
-	  "The show system processes command display information about the\n"
+	  "The `show system processes' command display information about the\n"
 	  "processes currently running on the system. The regular expression\n"
 	  "<regexp> is matched against the command executed by the process\n"
 	  "to select the processes of interest.The command generates a table\n"
@@ -1503,7 +1542,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_processes },
 
 	{ "monitor system storage", NULL,
-	  "The monitor system storage command shows the same\n"
+	  "The `monitor system storage' command shows the same\n"
 	  "information as the show system storage command. The\n"
 	  "information is updated periodically.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR | SCLI_CMD_FLAG_DRY,
@@ -1511,7 +1550,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_storage },
 #if 0
 	{ "loop system storage", NULL,
-	  "The monitor system storage command shows the same\n"
+	  "The `monitor system storage' command shows the same\n"
 	  "information as the show system storage command. The\n"
 	  "information is updated periodically.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_LOOP | SCLI_CMD_FLAG_XML,
@@ -1519,7 +1558,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  show_system_storage },
 #endif
 	{ "monitor system processes", "[<regexp>]",
-	  "The monitor system processes command show the same\n"
+	  "The `monitor system processes' command show the same\n"
 	  "information as the show system processes command. The\n"
 	  "information is updated periodically.",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR,
@@ -1534,7 +1573,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  cmd_xxx },
 #endif
 	{ "dump system", NULL,
-	  "The dump system command generates a sequence of scli commands\n"
+	  "The `dump system' command generates a sequence of scli commands\n"
 	  "which can be used to restore the system configuration.\n",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
