@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: vendors.c,v 1.10 2002/03/19 20:51:30 schoenw Exp $
+ * @(#) $Id: vendors.c,v 1.13 2002/07/22 19:43:38 schoenw Exp $
  */
 
 #include "scli.h"
@@ -29,12 +29,15 @@ static scli_vendor_t iana_vendor_table[] = {
     {   43,	"3Com",			"http://www.3com.com/" },
     {   45,	"SynOptics",		"http://www.synoptics.com/" },
     {   59,	"Silicon Graphics",	"http://www.sgi.com/" },
+    {   81,	"Avaya",		"http://www.avaya.com/" },
     {   82,	"Network Computing Devices",	"http://www.ncd.com/" },
     {  253,	"Xerox",		"http://www.xerox.com/" },
     {  311,	"Microsoft",		"http://www.microsoft.com/" },
     { 1575,	"TU Braunschweig",	NULL },
     { 1751,	"Lucent Technologies",	"http://www.lucent.com/" },
     { 2021,	"UCD-SNMP",		"http://ucd-snmp.ucdavis.edu/" },
+    { 2281,	"Ceragon",		"http://www.ceragon.com/" },
+    { 2356,	"ELSA GmbH",		"http://www.elsa.de/" },
     { 8072,	"NET-SNMP",		"http://net-snmp.sourceforge.net/" },
     {    0,	NULL,			NULL },
 };
@@ -87,8 +90,8 @@ scli_get_vendor_oid(guint32 oid[], gsize len)
  * The following list is taken from the Ethernet code list
  * available from <ftp://ftp.cavebear.com/pub/Ethernet-codes>:
  *
- * $Revision: 1.10 $
- * $Date: 2002/03/19 20:51:30 $
+ * $Revision: 1.13 $
+ * $Date: 2002/07/22 19:43:38 $
  *
  * The list was subsequently cleaned up and modified to suit the
  * needs of the scli package.
