@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: g_asn1.c,v 1.3 2001/08/27 07:52:30 schoenw Exp $
+ * $Id: g_asn1.c,v 1.5 2002/07/03 18:36:36 schoenw Exp $
  */
 
 /*
@@ -132,14 +132,13 @@ g_asn1_close(ASN1_SCK *asn1, guchar **buf, guint *len)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static inline gboolean 
 g_asn1_octet_encode(ASN1_SCK *asn1, guchar ch)
 {
-    if (asn1->pointer <= asn1->begin)
-	{
-		asn1->error = ASN1_ERR_ENC_FULL;
+    if (asn1->pointer <= asn1->begin) {
+	asn1->error = ASN1_ERR_ENC_FULL;
         return FALSE;
-	}
+    }
     *--(asn1->pointer) = ch;
     return TRUE;
 }
@@ -155,14 +154,13 @@ g_asn1_octet_encode(ASN1_SCK *asn1, guchar ch)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static inline gboolean 
 g_asn1_octet_decode(ASN1_SCK *asn1, guchar *ch)
 {
-    if (asn1->pointer >= asn1->end)
-	{
-		asn1->error = ASN1_ERR_DEC_EMPTY;
+    if (asn1->pointer >= asn1->end) {
+	asn1->error = ASN1_ERR_DEC_EMPTY;
         return FALSE;
-	}
+    }
     *ch = *(asn1->pointer)++;
     return TRUE;
 }
@@ -187,8 +185,7 @@ g_asn1_tag_encode(ASN1_SCK *asn1, guint tag)
     tag >>= 7;
     if (!g_asn1_octet_encode(asn1, ch))
         return FALSE;
-    while (tag > 0)
-    {
+    while (tag > 0) {
         ch = (guchar) (tag | 0x80);
         tag >>= 7;
         if (!g_asn1_octet_encode(asn1, ch))
@@ -214,8 +211,7 @@ g_asn1_tag_decode(ASN1_SCK *asn1, guint *tag)
     guchar ch;
 
     *tag = 0;
-    do
-    {
+    do {
         if (!g_asn1_octet_decode(asn1, &ch))
             return FALSE;
         *tag <<= 7;
@@ -243,8 +239,7 @@ g_asn1_id_encode(ASN1_SCK *asn1, guint cls, guint con, guint tag)
 {
     guint ch;
 
-    if (tag >= 0x1F)
-    {
+    if (tag >= 0x1F) {
         if (!g_asn1_tag_encode (asn1, tag))
             return FALSE;
         tag = 0x1F;
@@ -303,17 +298,14 @@ g_asn1_length_encode(ASN1_SCK *asn1, guint def, guint len)
 {
     guchar ch, cnt;
 
-    if (!def)
+    if (!def) {
         ch = 0x80;
-    else
-    {
-        if (len < 0x80)
+    } else {
+        if (len < 0x80) {
             ch = (guchar) len;
-        else
-        {
+	} else {
             cnt = 0;
-            while (len > 0)
-            {
+            while (len > 0) {
                 ch = (guchar) len;
                 len >>= 8;
                 if (!g_asn1_octet_encode (asn1, ch))
@@ -344,22 +336,19 @@ gboolean
 g_asn1_length_decode(ASN1_SCK *asn1, guint *def, guint *len)
 {
     guchar ch, cnt;
-
+    
     if (!g_asn1_octet_decode (asn1, &ch))
         return FALSE;
-    if (ch == 0x80)
+    if (ch == 0x80) {
         *def = 0;
-    else
-    {
+    } else {
         *def = 1;
-        if (ch < 0x80)
+        if (ch < 0x80) {
             *len = ch;
-        else
-        {
+	} else {
             cnt = (guchar) (ch & 0x7F);
             *len = 0;
-            while (cnt > 0)
-            {
+            while (cnt > 0) {
                 if (!g_asn1_octet_decode (asn1, &ch))
                     return FALSE;
                 *len <<= 8;
@@ -398,13 +387,10 @@ g_asn1_header_encode(ASN1_SCK *asn1, guchar *eoc, guint cls,
 {
     guint def, len;
 
-    if (eoc == 0)
-    {
+    if (eoc == 0) {
         def = 0;
         len = 0;
-    }
-    else
-    {
+    } else {
         def = 1;
         len = eoc - asn1->pointer;
     }
@@ -412,6 +398,9 @@ g_asn1_header_encode(ASN1_SCK *asn1, guchar *eoc, guint cls,
         return FALSE;
     if (!g_asn1_id_encode (asn1, cls, con, tag))
         return FALSE;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	g_printerr("asn1enc %p: SEQUENCE (OF)\n", asn1);
+    }
     return TRUE;
 }
 
@@ -450,6 +439,9 @@ g_asn1_header_decode(ASN1_SCK *asn1, guchar **eoc, guint *cls,
         *eoc = asn1->pointer + len;
     else
         *eoc = 0;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1 && *con) {
+	g_printerr("asn1dec %p: SEQUENCE (OF)\n", asn1);
+    }
     return TRUE;
 }
 
@@ -472,10 +464,11 @@ g_asn1_header_decode(ASN1_SCK *asn1, guchar **eoc, guint *cls,
 gboolean 
 g_asn1_eoc ( ASN1_SCK *asn1, guchar *eoc)
 {
-  if (eoc == 0)
-    return (asn1->pointer [0] == 0x00 && asn1->pointer [1] == 0x00);
-  else
-    return (asn1->pointer >= eoc);
+    if (eoc == 0) {
+	return (asn1->pointer [0] == 0x00 && asn1->pointer [1] == 0x00);
+    } else {
+	return (asn1->pointer >= eoc);
+    }
 }
 
 /*
@@ -504,18 +497,15 @@ g_asn1_eoc ( ASN1_SCK *asn1, guchar *eoc)
 gboolean 
 g_asn1_eoc_encode ( ASN1_SCK *asn1, guchar **eoc )        
 {
-  if (eoc == 0)
-    {
-      if (!g_asn1_octet_encode (asn1, 0x00))
-	return FALSE;
-      if (!g_asn1_octet_encode (asn1, 0x00))
-	return FALSE;
-      return TRUE;
-    }
-  else
-    {
-      *eoc = asn1->pointer;
-      return TRUE;
+    if (eoc == 0) {
+	if (!g_asn1_octet_encode (asn1, 0x00))
+	    return FALSE;
+	if (!g_asn1_octet_encode (asn1, 0x00))
+	    return FALSE;
+	return TRUE;
+    } else {
+	*eoc = asn1->pointer;
+	return TRUE;
     }
 }
 
@@ -546,34 +536,28 @@ g_asn1_eoc_encode ( ASN1_SCK *asn1, guchar **eoc )
 gboolean 
 g_asn1_eoc_decode (ASN1_SCK *asn1, guchar   *eoc)
 {
-  guchar ch;
+    guchar ch;
     
-  if (eoc == 0)
-    {
-      if (!g_asn1_octet_decode (asn1, &ch))
-	return FALSE;
-      if (ch != 0x00)
-	{
-	  asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
-	  return FALSE;
+    if (eoc == 0) {
+	if (!g_asn1_octet_decode (asn1, &ch))
+	    return FALSE;
+	if (ch != 0x00) {
+	    asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
+	    return FALSE;
 	}
-      if (!g_asn1_octet_decode (asn1, &ch))
-	return FALSE;
-      if (ch != 0x00)
-	{
-	  asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
-	  return FALSE;
+	if (!g_asn1_octet_decode (asn1, &ch))
+	    return FALSE;
+	if (ch != 0x00) {
+	    asn1->error = ASN1_ERR_DEC_EOC_MISMATCH;
+	    return FALSE;
 	}
-      return TRUE;
-    }
-  else
-    {
-      if (asn1->pointer != eoc)
-	{
-	  asn1->error = ASN1_ERR_DEC_LENGTH_MISMATCH;
-	  return FALSE;
+	return TRUE;
+    } else {
+	if (asn1->pointer != eoc) {
+	    asn1->error = ASN1_ERR_DEC_LENGTH_MISMATCH;
+	    return FALSE;
 	}
-      return TRUE;
+	return TRUE;
     }
 }
 
@@ -595,8 +579,11 @@ g_asn1_eoc_decode (ASN1_SCK *asn1, guchar   *eoc)
 gboolean 
 g_asn1_null_encode ( ASN1_SCK *asn1, guchar **eoc )
 {
-  *eoc = asn1->pointer;
-  return TRUE;
+    *eoc = asn1->pointer;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	g_printerr("asn1enc %p: NULL\n", asn1);
+    }
+    return TRUE;
 }
 
 /*
@@ -618,8 +605,11 @@ g_asn1_null_encode ( ASN1_SCK *asn1, guchar **eoc )
 gboolean 
 g_asn1_null_decode ( ASN1_SCK *asn1, guchar *eoc)
 {
-  asn1->pointer = eoc;
-  return TRUE;
+    asn1->pointer = eoc;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	g_printerr("asn1dec %p: NULL\n", asn1);
+    }
+    return TRUE;
 }
 
 /*
@@ -643,26 +633,21 @@ g_asn1_int32_encode ( ASN1_SCK *asn1, guchar **eoc, gint32 integer)
 {
     guchar ch,sign;
     int    lim;
-
+    
     *eoc = asn1->pointer;
-    if (integer < 0)
-    {
+    if (integer < 0) {
         lim  = -1;
         sign = 0x80;
-    }
-    else
-    {
+    } else {
         lim  = 0;
         sign = 0x00;
     }
-    do
-    {
+    do {
         ch = (guchar) integer;
         integer >>= 8;
         if (!g_asn1_octet_encode (asn1, ch))
             return FALSE;
-    }
-    while ((integer != lim) || (guchar) (ch & 0x80) != sign);
+    } while ((integer != lim) || (guchar) (ch & 0x80) != sign);
     return TRUE;
 }
 
@@ -688,25 +673,23 @@ g_asn1_int32_decode ( ASN1_SCK *asn1, guchar *eoc, gint32 *integer)
 {
     guchar ch;
     guint  len;
-
+    
     if (!g_asn1_octet_decode (asn1, &ch))
         return FALSE;
     *integer = (gchar) ch;
     len = 1;
-    while (asn1->pointer < eoc)
-    {
-        if (++len > sizeof (gint32))
-		{
-			asn1->error = ASN1_ERR_DEC_BADVALUE;
+    while (asn1->pointer < eoc) {
+        if (++len > sizeof (gint32)) {
+	    asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
-		}
+	}
         if (!g_asn1_octet_decode (asn1, &ch))
             return FALSE;
         *integer <<= 8;
         *integer |= ch;
     }
     if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
-	g_printerr("asn1: INTEGER %d\n", *integer);
+	g_printerr("asn1dec %p: INTEGER %d\n", asn1, *integer);
     }
     return TRUE;
 }
@@ -732,26 +715,21 @@ g_asn1_int64_encode ( ASN1_SCK *asn1, guchar **eoc, gint64 integer)
 {
     guchar ch, sign;
     glong  lim;
-
+    
     *eoc = asn1->pointer;
-    if (integer < 0)
-    {
+    if (integer < 0) {
         lim  = -1;
         sign = 0x80;
-    }
-    else
-    {
+    } else {
         lim  = 0;
         sign = 0x00;
     }
-    do
-    {
+    do {
         ch = (guchar) integer;
         integer >>= 8;
         if (!g_asn1_octet_encode (asn1, ch))
             return FALSE;
-    }
-    while ((integer != lim) || (guchar) (ch & 0x80) != sign);
+    } while ((integer != lim) || (guchar) (ch & 0x80) != sign);
     return TRUE;
 }
 
@@ -782,17 +760,15 @@ g_asn1_int64_decode ( ASN1_SCK *asn1, guchar *eoc, gint64 *integer)
         return FALSE;
     *integer = (gchar) ch;
     len = 1;
-    while (asn1->pointer < eoc)
-    {
-      if (++len > sizeof (gint64))
-	{
-	  asn1->error = ASN1_ERR_DEC_BADVALUE;
-	  return FALSE;
+    while (asn1->pointer < eoc) {
+	if (++len > sizeof (gint64)) {
+	    asn1->error = ASN1_ERR_DEC_BADVALUE;
+	    return FALSE;
 	}
-      if (!g_asn1_octet_decode (asn1, &ch))
-	return FALSE;
-      *integer <<= 8;
-      *integer |= ch;
+	if (!g_asn1_octet_decode (asn1, &ch))
+	    return FALSE;
+	*integer <<= 8;
+	*integer |= ch;
     }
     return TRUE;
 }
@@ -816,18 +792,16 @@ g_asn1_int64_decode ( ASN1_SCK *asn1, guchar *eoc, gint64 *integer)
 gboolean 
 g_asn1_uint32_encode ( ASN1_SCK *asn1, guchar **eoc, guint32 integer)
 {
-  guchar ch;
-  
-  *eoc = asn1->pointer;
-  do
-    {
-      ch = (guchar) integer;
-      integer >>= 8;
-      if (!g_asn1_octet_encode (asn1, ch))
-	return FALSE;
-    }
-  while ((integer != 0) || (ch & 0x80) != 0x00);
-  return TRUE;
+    guchar ch;
+    
+    *eoc = asn1->pointer;
+    do {
+	ch = (guchar) integer;
+	integer >>= 8;
+	if (!g_asn1_octet_encode (asn1, ch))
+	    return FALSE;
+    } while ((integer != 0) || (ch & 0x80) != 0x00);
+    return TRUE;
 }
 
 /*
@@ -852,24 +826,24 @@ g_asn1_uint32_decode ( ASN1_SCK *asn1, guchar *eoc, guint32 *integer)
 {
     guchar ch;
     guint  len;
-
+    
     if (!g_asn1_octet_decode (asn1, &ch))
         return FALSE;
     *integer = ch;
-    if (ch == 0) len = 0;
-    else len = 1;
-    while (asn1->pointer < eoc)
-    {
-        if (++len > sizeof (guint32))
-		{
-			asn1->error = ASN1_ERR_DEC_BADVALUE;
+    len = (ch == 0) ? 0 : 1;
+    while (asn1->pointer < eoc) {
+        if (++len > sizeof (guint32)) {
+	    asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
-		}
+	}
         if (!g_asn1_octet_decode (asn1, &ch))
             return FALSE;
         *integer <<= 8;
         *integer |= ch;
     }
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	g_printerr("asn1dec %p: INTEGER %u\n", asn1, *integer);
+    }
     return TRUE;
 }
 
@@ -895,8 +869,7 @@ g_asn1_uint64_encode ( ASN1_SCK *asn1, guchar **eoc, guint64 integer)
     guchar ch;
 
     *eoc = asn1->pointer;
-    do
-    {
+    do {
         ch = (guchar) integer;
         integer >>= 8;
         if (!g_asn1_octet_encode (asn1, ch))
@@ -932,17 +905,12 @@ g_asn1_uint64_decode ( ASN1_SCK *asn1, guchar *eoc, guint64 *integer)
     if (!g_asn1_octet_decode (asn1, &ch))
         return FALSE;
     *integer = ch;
-    if (ch == 0)
-        len = 0;
-    else
-        len = 1;
-    while (asn1->pointer < eoc)
-    {
-        if (++len > sizeof (guint64))
-		{
-			asn1->error = ASN1_ERR_DEC_BADVALUE;
+    len = (ch == 0) ? 0 : 1;
+    while (asn1->pointer < eoc) {
+        if (++len > sizeof (guint64)) {
+	    asn1->error = ASN1_ERR_DEC_BADVALUE;
             return FALSE;
-		}
+	}
         if (!g_asn1_octet_decode (asn1, &ch))
             return FALSE;
         *integer <<= 8;
@@ -971,17 +939,16 @@ g_asn1_uint64_decode ( ASN1_SCK *asn1, guchar *eoc, guint64 *integer)
 gboolean 
 g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets, guint len)
 {
-  guchar *ptr;
+    guchar *ptr;
 
-  *eoc = asn1->pointer;
-  ptr = octets + len;
-
-  while (len-- > 0)
-    {
-      if (!g_asn1_octet_encode (asn1, *--ptr))
-        return FALSE;
+    *eoc = asn1->pointer;
+    ptr = octets + len;
+    
+    while (len-- > 0) {
+	if (!g_asn1_octet_encode (asn1, *--ptr))
+	    return FALSE;
     }
-  return TRUE;
+    return TRUE;
 }
 
 /*
@@ -1005,23 +972,29 @@ g_asn1_octets_encode ( ASN1_SCK *asn1, guchar **eoc, guchar *octets, guint len)
 gboolean 
 g_asn1_octets_decode ( ASN1_SCK *asn1, guchar *eoc, guchar **octets, guint *len)
 {
-  guchar *ptr;
-
-  *octets = NULL;
-  *len = 0;
-  *octets = g_malloc (eoc - asn1->pointer);
-  ptr = *octets;
-  while (asn1->pointer < eoc)
-    {
-      if (!g_asn1_octet_decode (asn1, (guchar *)ptr++))
-        {
-          g_free(*octets);
-          *octets = NULL;
-	  return FALSE;
-        }
-      (*len)++;
+    guchar *ptr;
+    
+    *octets = NULL;
+    *len = 0;
+    *octets = g_malloc(eoc - asn1->pointer);
+    ptr = *octets;
+    while (asn1->pointer < eoc) {
+	if (!g_asn1_octet_decode (asn1, (guchar *)ptr++)) {
+	    g_free(*octets);
+	    *octets = NULL;
+	    return FALSE;
+	}
+	(*len)++;
     }
-  return TRUE;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	guint i;
+	g_printerr("asn1dec %p: OCTET STRING '", asn1);
+	for (i = 0; i < *len; i++) {
+	    g_printerr("%02x", (*octets)[i]);
+	}
+	g_printerr("'h\n");
+    }
+    return TRUE;
 }
 
 /*
@@ -1038,23 +1011,22 @@ g_asn1_octets_decode ( ASN1_SCK *asn1, guchar *eoc, guchar **octets, guint *len)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static inline gboolean 
 g_asn1_subid_encode ( ASN1_SCK *asn1, guint32 subid)
 {
-  guchar ch;
-  
-  ch = (guchar) (subid & 0x7F);
-  subid >>= 7;
-  if (!g_asn1_octet_encode (asn1, ch))
-    return FALSE;
-  while (subid > 0)
-    {
-      ch = (guchar) (subid | 0x80);
-      subid >>= 7;
-      if (!g_asn1_octet_encode (asn1, ch))
+    guchar ch;
+    
+    ch = (guchar) (subid & 0x7F);
+    subid >>= 7;
+    if (!g_asn1_octet_encode (asn1, ch))
 	return FALSE;
+    while (subid > 0) {
+	ch = (guchar) (subid | 0x80);
+	subid >>= 7;
+	if (!g_asn1_octet_encode (asn1, ch))
+	    return FALSE;
     }
-  return TRUE;
+    return TRUE;
 }
 
 /*
@@ -1071,14 +1043,13 @@ g_asn1_subid_encode ( ASN1_SCK *asn1, guint32 subid)
  * RETURNS:     gboolean success.
  */
 
-gboolean 
+static inline gboolean 
 g_asn1_subid_decode ( ASN1_SCK *asn1, guint32 *subid)
 {
     guchar ch;
-
+    
     *subid = 0;
-    do
-    {
+    do {
         if (!g_asn1_octet_decode(asn1, &ch))
             return FALSE;
         *subid <<= 7;
@@ -1110,24 +1081,31 @@ gboolean
 g_asn1_oid_encode ( ASN1_SCK *asn1, guchar **eoc, guint32 *oid,
 		    guint len)
 {
-  gulong subid;
-  
-  *eoc = asn1->pointer;
-  if (len < 2)
-    {
-      asn1->error = ASN1_ERR_ENC_BADVALUE;
-      return FALSE;
-    }
-  subid = oid [1] + oid [0] * 40;
-  oid += len;
-  while (len-- > 2)
-    {
-      if (!g_asn1_subid_encode (asn1, *--oid))
+    gulong subid;
+    guint l = len;
+    
+    *eoc = asn1->pointer;
+    if (len < 2) {
+	asn1->error = ASN1_ERR_ENC_BADVALUE;
 	return FALSE;
     }
-  if (!g_asn1_subid_encode (asn1, subid))
-    return FALSE;
-  return TRUE;
+    subid = oid [1] + oid [0] * 40;
+    oid += len;
+    while (l-- > 2) {
+	if (!g_asn1_subid_encode (asn1, *--oid))
+	    return FALSE;
+    }
+    if (!g_asn1_subid_encode (asn1, subid))
+	return FALSE;
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	guint i;
+	g_printerr("asn1enc %p: OBJECT IDENTIFIER ", asn1);
+	for (i = 0; i < len; i++) {
+	    g_printerr("%s%u", i ? "." : "", oid[i]);
+	}
+	g_printerr("\n");
+    }
+    return TRUE;
 }
 
 /*
@@ -1154,63 +1132,54 @@ gboolean
 g_asn1_oid_decode ( ASN1_SCK *asn1, guchar *eoc, guint32 **oid, 
 		    guint *len)
 {
-  guint32 subid;
-  guint  size;
-  guint32 *optr;
-
-  size = eoc - asn1->pointer + 1;
-  *oid = g_malloc(size * sizeof(guint32));
-  optr = *oid;
- 
-  if (!g_asn1_subid_decode (asn1, &subid))
-    {
-      g_free(*oid);
-      *oid = NULL;
-      return FALSE;
+    guint32 subid;
+    guint  size;
+    guint32 *optr;
+    
+    size = eoc - asn1->pointer + 1;
+    *oid = g_malloc(size * sizeof(guint32));
+    optr = *oid;
+    
+    if (!g_asn1_subid_decode (asn1, &subid)) {
+	g_free(*oid);
+	*oid = NULL;
+	return FALSE;
     }
-  if (subid < 40)
-    {
-      optr [0] = 0;
-      optr [1] = subid;
+    if (subid < 40) {
+	optr [0] = 0;
+	optr [1] = subid;
+    } else if (subid < 80) {
+	optr [0] = 1;
+	optr [1] = subid - 40;
+    } else {
+	optr [0] = 2;
+	optr [1] = subid - 80;
     }
-  else if (subid < 80)
-    {
-      optr [0] = 1;
-      optr [1] = subid - 40;
-    }
-  else
-    {
-      optr [0] = 2;
-      optr [1] = subid - 80;
-    }
-  *len = 2;
-  optr += 2;
-  while (asn1->pointer < eoc)
-    {
-      if (++(*len) > size)
-	{
-	  asn1->error = ASN1_ERR_DEC_BADVALUE;
-          g_free(*oid);
-          *oid = NULL;
-	  return FALSE;
+    *len = 2;
+    optr += 2;
+    while (asn1->pointer < eoc) {
+	if (++(*len) > size) {
+	    asn1->error = ASN1_ERR_DEC_BADVALUE;
+	    g_free(*oid);
+	    *oid = NULL;
+	    return FALSE;
 	}
-      if (!g_asn1_subid_decode (asn1, optr++))
-        {
-          g_free(*oid);
-          *oid = NULL;
-	  return FALSE;
+	if (!g_asn1_subid_decode (asn1, optr++)) {
+	    g_free(*oid);
+	    *oid = NULL;
+	    return FALSE;
         }
     }
-
-  if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
-      int i;
-      g_printerr("asn1: OBJECT IDENTIFIER ");
-      for (i = 0; i < *len; i++) {
-	  g_printerr("%s%u", i ? "." : "", (*oid)[i]);
-      }
-      g_printerr("\n");
-  }
-  return TRUE;
+    
+    if (g_snmp_debug_flags & G_SNMP_DEBUG_ASN1) {
+	guint i;
+	g_printerr("asn1dec %p: OBJECT IDENTIFIER ", asn1);
+	for (i = 0; i < *len; i++) {
+	    g_printerr("%s%u", i ? "." : "", (*oid)[i]);
+	}
+	g_printerr("\n");
+    }
+    return TRUE;
 }
 
 /* EOF */
